# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import MutableMapping, MutableSequence

import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.aiplatform.v1.schema.trainingjob.definition",
    manifest={
        "AutoMlImageClassification",
        "AutoMlImageClassificationInputs",
        "AutoMlImageClassificationMetadata",
    },
)


class AutoMlImageClassification(proto.Message):
    r"""A TrainingJob that trains and uploads an AutoML Image
    Classification Model.

    Attributes:
        inputs (google.cloud.aiplatform.v1.schema.trainingjob.definition_v1.types.AutoMlImageClassificationInputs):
            The input parameters of this TrainingJob.
        metadata (google.cloud.aiplatform.v1.schema.trainingjob.definition_v1.types.AutoMlImageClassificationMetadata):
            The metadata information.
    """

    inputs: "AutoMlImageClassificationInputs" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="AutoMlImageClassificationInputs",
    )
    metadata: "AutoMlImageClassificationMetadata" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="AutoMlImageClassificationMetadata",
    )


class AutoMlImageClassificationInputs(proto.Message):
    r"""

    Attributes:
        model_type (google.cloud.aiplatform.v1.schema.trainingjob.definition_v1.types.AutoMlImageClassificationInputs.ModelType):

        base_model_id (str):
            The ID of the ``base`` model. If it is specified, the new
            model will be trained based on the ``base`` model.
            Otherwise, the new model will be trained from scratch. The
            ``base`` model must be in the same Project and Location as
            the new Model to train, and have the same modelType.
        budget_milli_node_hours (int):
            The training budget of creating this model, expressed in
            milli node hours i.e. 1,000 value in this field means 1 node
            hour. The actual metadata.costMilliNodeHours will be equal
            or less than this value. If further model training ceases to
            provide any improvements, it will stop without using the
            full budget and the metadata.successfulStopReason will be
            ``model-converged``. Note, node_hour = actual_hour \*
            number_of_nodes_involved. For modelType
            ``cloud``\ (default), the budget must be between 8,000 and
            800,000 milli node hours, inclusive. The default value is
            192,000 which represents one day in wall time, considering 8
            nodes are used. For model types ``mobile-tf-low-latency-1``,
            ``mobile-tf-versatile-1``, ``mobile-tf-high-accuracy-1``,
            the training budget must be between 1,000 and 100,000 milli
            node hours, inclusive. The default value is 24,000 which
            represents one day in wall time on a single node that is
            used.
        disable_early_stopping (bool):
            Use the entire training budget. This disables
            the early stopping feature. When false the early
            stopping feature is enabled, which means that
            AutoML Image Classification might stop training
            before the entire training budget has been used.
        multi_label (bool):
            If false, a single-label (multi-class) Model
            will be trained (i.e. assuming that for each
            image just up to one annotation may be
            applicable). If true, a multi-label Model will
            be trained (i.e. assuming that for each image
            multiple annotations may be applicable).
    """

    class ModelType(proto.Enum):
        r"""

        Values:
            MODEL_TYPE_UNSPECIFIED (0):
                Should not be set.
            CLOUD (1):
                A Model best tailored to be used within
                Google Cloud, and which cannot be exported.
                Default.
            MOBILE_TF_LOW_LATENCY_1 (2):
                A model that, in addition to being available
                within Google Cloud, can also be exported (see
                ModelService.ExportModel) as TensorFlow or Core
                ML model and used on a mobile or edge device
                afterwards. Expected to have low latency, but
                may have lower prediction quality than other
                mobile models.
            MOBILE_TF_VERSATILE_1 (3):
                A model that, in addition to being available
                within Google Cloud, can also be exported (see
                ModelService.ExportModel) as TensorFlow or Core
                ML model and used on a mobile or edge device
                with afterwards.
            MOBILE_TF_HIGH_ACCURACY_1 (4):
                A model that, in addition to being available
                within Google Cloud, can also be exported (see
                ModelService.ExportModel) as TensorFlow or Core
                ML model and used on a mobile or edge device
                afterwards. Expected to have a higher latency,
                but should also have a higher prediction quality
                than other mobile models.
        """
        MODEL_TYPE_UNSPECIFIED = 0
        CLOUD = 1
        MOBILE_TF_LOW_LATENCY_1 = 2
        MOBILE_TF_VERSATILE_1 = 3
        MOBILE_TF_HIGH_ACCURACY_1 = 4

    model_type: ModelType = proto.Field(
        proto.ENUM,
        number=1,
        enum=ModelType,
    )
    base_model_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    budget_milli_node_hours: int = proto.Field(
        proto.INT64,
        number=3,
    )
    disable_early_stopping: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    multi_label: bool = proto.Field(
        proto.BOOL,
        number=5,
    )


class AutoMlImageClassificationMetadata(proto.Message):
    r"""

    Attributes:
        cost_milli_node_hours (int):
            The actual training cost of creating this
            model, expressed in milli node hours, i.e. 1,000
            value in this field means 1 node hour.
            Guaranteed to not exceed
            inputs.budgetMilliNodeHours.
        successful_stop_reason (google.cloud.aiplatform.v1.schema.trainingjob.definition_v1.types.AutoMlImageClassificationMetadata.SuccessfulStopReason):
            For successful job completions, this is the
            reason why the job has finished.
    """

    class SuccessfulStopReason(proto.Enum):
        r"""

        Values:
            SUCCESSFUL_STOP_REASON_UNSPECIFIED (0):
                Should not be set.
            BUDGET_REACHED (1):
                The inputs.budgetMilliNodeHours had been
                reached.
            MODEL_CONVERGED (2):
                Further training of the Model ceased to
                increase its quality, since it already has
                converged.
        """
        SUCCESSFUL_STOP_REASON_UNSPECIFIED = 0
        BUDGET_REACHED = 1
        MODEL_CONVERGED = 2

    cost_milli_node_hours: int = proto.Field(
        proto.INT64,
        number=1,
    )
    successful_stop_reason: SuccessfulStopReason = proto.Field(
        proto.ENUM,
        number=2,
        enum=SuccessfulStopReason,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import copy
from importlib import reload
from unittest import mock
from unittest.mock import patch, call

import numpy as np
from sklearn.linear_model import LinearRegression

import pytest
from google.api_core import exceptions
from google.api_core import operation
from google.auth import credentials

import google.cloud.aiplatform.metadata.constants
from google.cloud.aiplatform.metadata.schema.google import (
    artifact_schema as google_artifact_schema,
)
from google.cloud import aiplatform
from google.cloud.aiplatform import base
from google.cloud.aiplatform import initializer
from google.cloud.aiplatform_v1 import (
    AddContextArtifactsAndExecutionsResponse,
    LineageSubgraph,
    Artifact as GapicArtifact,
    Context as GapicContext,
    Execution as GapicExecution,
    MetadataServiceClient,
    AddExecutionEventsResponse,
    MetadataStore as GapicMetadataStore,
    TensorboardServiceClient,
)
from google.cloud.aiplatform.compat.types import event as gca_event
from google.cloud.aiplatform.compat.types import execution as gca_execution
from google.cloud.aiplatform.compat.types import (
    tensorboard_data as gca_tensorboard_data,
)
from google.cloud.aiplatform.compat.types import (
    tensorboard_experiment as gca_tensorboard_experiment,
)
from google.cloud.aiplatform.compat.types import (
    tensorboard_run as gca_tensorboard_run,
)
from google.cloud.aiplatform.compat.types import (
    tensorboard_time_series as gca_tensorboard_time_series,
)
from google.cloud.aiplatform.metadata import constants
from google.cloud.aiplatform.metadata import experiment_run_resource
from google.cloud.aiplatform.metadata import metadata
from google.cloud.aiplatform.metadata import metadata_store
from google.cloud.aiplatform.metadata import utils as metadata_utils
from google.cloud.aiplatform import utils

import constants as test_constants

_TEST_PROJECT = "test-project"
_TEST_OTHER_PROJECT = "test-project-1"
_TEST_LOCATION = "us-central1"
_TEST_PARENT = (
    f"projects/{_TEST_PROJECT}/locations/{_TEST_LOCATION}/metadataStores/default"
)
_TEST_EXPERIMENT = "test-experiment"
_TEST_EXPERIMENT_DESCRIPTION = "test-experiment-description"
_TEST_OTHER_EXPERIMENT_DESCRIPTION = "test-other-experiment-description"
_TEST_PIPELINE = _TEST_EXPERIMENT
_TEST_RUN = "run-1"
_TEST_OTHER_RUN = "run-2"
_TEST_DISPLAY_NAME = "test-display-name"
_TEST_CREDENTIALS = mock.Mock(spec=credentials.AnonymousCredentials())
_TEST_BUCKET_NAME = "gs://test-bucket"

# resource attributes
_TEST_METADATA = {"test-param1": 1, "test-param2": "test-value", "test-param3": True}

# metadataStore
_TEST_METADATASTORE = (
    f"projects/{_TEST_PROJECT}/locations/{_TEST_LOCATION}/metadataStores/default"
)

# context
_TEST_CONTEXT_ID = _TEST_EXPERIMENT
_TEST_CONTEXT_NAME = f"{_TEST_PARENT}/contexts/{_TEST_CONTEXT_ID}"

# execution
_TEST_EXECUTION_ID = f"{_TEST_EXPERIMENT}-{_TEST_RUN}"
_TEST_EXECUTION_NAME = f"{_TEST_PARENT}/executions/{_TEST_EXECUTION_ID}"
_TEST_OTHER_EXECUTION_ID = f"{_TEST_EXPERIMENT}-{_TEST_OTHER_RUN}"
_TEST_OTHER_EXECUTION_NAME = f"{_TEST_PARENT}/executions/{_TEST_OTHER_EXECUTION_ID}"
_TEST_SCHEMA_TITLE = "test.Schema"

_TEST_EXECUTION = GapicExecution(
    name=_TEST_EXECUTION_NAME,
    schema_title=_TEST_SCHEMA_TITLE,
    display_name=_TEST_DISPLAY_NAME,
    metadata=_TEST_METADATA,
    state=GapicExecution.State.RUNNING,
)

# artifact
_TEST_ARTIFACT_ID = f"{_TEST_EXPERIMENT}-{_TEST_RUN}-metrics"
_TEST_ARTIFACT_NAME = f"{_TEST_PARENT}/artifacts/{_TEST_ARTIFACT_ID}"
_TEST_OTHER_ARTIFACT_ID = f"{_TEST_EXPERIMENT}-{_TEST_OTHER_RUN}-metrics"
_TEST_OTHER_ARTIFACT_NAME = f"{_TEST_PARENT}/artifacts/{_TEST_OTHER_ARTIFACT_ID}"
_TEST_MODEL_ID = "test-model"
_TEST_MODEL_NAME = f"{_TEST_PARENT}/artifacts/{_TEST_MODEL_ID}"

# parameters
_TEST_PARAM_KEY_1 = "learning_rate"
_TEST_PARAM_KEY_2 = "dropout"
_TEST_PARAMS = {_TEST_PARAM_KEY_1: 0.01, _TEST_PARAM_KEY_2: 0.2}
_TEST_OTHER_PARAMS = {_TEST_PARAM_KEY_1: 0.02, _TEST_PARAM_KEY_2: 0.3}

# metrics
_TEST_METRIC_KEY_1 = "rmse"
_TEST_METRIC_KEY_2 = "accuracy"
_TEST_METRICS = {_TEST_METRIC_KEY_1: 222, _TEST_METRIC_KEY_2: 1}
_TEST_OTHER_METRICS = {_TEST_METRIC_KEY_2: 0.9}

# classification_metrics
_TEST_CLASSIFICATION_METRICS = {
    "display_name": "my-classification-metrics",
    "labels": ["cat", "dog"],
    "matrix": [[9, 1], [1, 9]],
    "fpr": [0.1, 0.5, 0.9],
    "tpr": [0.1, 0.7, 0.9],
    "threshold": [0.9, 0.5, 0.1],
}

# schema
_TEST_WRONG_SCHEMA_TITLE = "system.WrongSchema"


@pytest.fixture
def get_metadata_store_mock():
    with patch.object(
        MetadataServiceClient, "get_metadata_store"
    ) as get_metadata_store_mock:
        get_metadata_store_mock.return_value = GapicMetadataStore(
            name=_TEST_METADATASTORE,
        )
        yield get_metadata_store_mock


@pytest.fixture
def get_metadata_store_mock_raise_not_found_exception():
    with patch.object(
        MetadataServiceClient, "get_metadata_store"
    ) as get_metadata_store_mock:
        get_metadata_store_mock.side_effect = [
            exceptions.NotFound("Test store not found."),
            GapicMetadataStore(
                name=_TEST_METADATASTORE,
            ),
        ]

        yield get_metadata_store_mock


@pytest.fixture
def create_metadata_store_mock():
    with patch.object(
        MetadataServiceClient, "create_metadata_store"
    ) as create_metadata_store_mock:
        create_metadata_store_lro_mock = mock.Mock(operation.Operation)
        create_metadata_store_lro_mock.result.return_value = GapicMetadataStore(
            name=_TEST_METADATASTORE,
        )
        create_metadata_store_mock.return_value = create_metadata_store_lro_mock
        yield create_metadata_store_mock


@pytest.fixture
def get_context_mock():
    with patch.object(MetadataServiceClient, "get_context") as get_context_mock:
        get_context_mock.return_value = GapicContext(
            name=_TEST_CONTEXT_NAME,
            display_name=_TEST_EXPERIMENT,
            description=_TEST_EXPERIMENT_DESCRIPTION,
            schema_title=constants.SYSTEM_EXPERIMENT,
            schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_EXPERIMENT],
            metadata=constants.EXPERIMENT_METADATA,
        )
        yield get_context_mock


@pytest.fixture
def get_context_wrong_schema_mock():
    with patch.object(
        MetadataServiceClient, "get_context"
    ) as get_context_wrong_schema_mock:
        get_context_wrong_schema_mock.return_value = GapicContext(
            name=_TEST_CONTEXT_NAME,
            display_name=_TEST_EXPERIMENT,
            schema_title=_TEST_WRONG_SCHEMA_TITLE,
            schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_EXPERIMENT],
            metadata=constants.EXPERIMENT_METADATA,
        )
        yield get_context_wrong_schema_mock


@pytest.fixture
def get_pipeline_context_mock():
    with patch.object(
        MetadataServiceClient, "get_context"
    ) as get_pipeline_context_mock:
        get_pipeline_context_mock.return_value = GapicContext(
            name=_TEST_CONTEXT_NAME,
            display_name=_TEST_EXPERIMENT,
            schema_title=constants.SYSTEM_PIPELINE,
            schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_PIPELINE],
            metadata=constants.EXPERIMENT_METADATA,
        )
        yield get_pipeline_context_mock


@pytest.fixture
def get_context_not_found_mock():
    with patch.object(
        MetadataServiceClient, "get_context"
    ) as get_context_not_found_mock:
        get_context_not_found_mock.side_effect = exceptions.NotFound("test: not found")
        yield get_context_not_found_mock


_TEST_EXPERIMENT_CONTEXT = GapicContext(
    name=_TEST_CONTEXT_NAME,
    display_name=_TEST_EXPERIMENT,
    description=_TEST_EXPERIMENT_DESCRIPTION,
    schema_title=constants.SYSTEM_EXPERIMENT,
    schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_EXPERIMENT],
    metadata={
        **constants.EXPERIMENT_METADATA,
        constants._BACKING_TENSORBOARD_RESOURCE_KEY: test_constants.TensorboardConstants._TEST_TENSORBOARD_NAME,
    },
)


@pytest.fixture
def update_context_mock():
    with patch.object(MetadataServiceClient, "update_context") as update_context_mock:
        update_context_mock.return_value = _TEST_EXPERIMENT_CONTEXT
        yield update_context_mock


@pytest.fixture
def add_context_artifacts_and_executions_mock():
    with patch.object(
        MetadataServiceClient, "add_context_artifacts_and_executions"
    ) as add_context_artifacts_and_executions_mock:
        add_context_artifacts_and_executions_mock.return_value = (
            AddContextArtifactsAndExecutionsResponse()
        )
        yield add_context_artifacts_and_executions_mock


@pytest.fixture
def get_execution_mock():
    with patch.object(MetadataServiceClient, "get_execution") as get_execution_mock:
        get_execution_mock.return_value = GapicExecution(
            name=_TEST_EXECUTION_NAME,
            display_name=_TEST_RUN,
            schema_title=constants.SYSTEM_RUN,
            schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_RUN],
        )
        yield get_execution_mock


@pytest.fixture
def get_execution_wrong_schema_mock():
    with patch.object(
        MetadataServiceClient, "get_execution"
    ) as get_execution_wrong_schema_mock:
        get_execution_wrong_schema_mock.return_value = GapicExecution(
            name=_TEST_EXECUTION_NAME,
            display_name=_TEST_RUN,
            schema_title=_TEST_WRONG_SCHEMA_TITLE,
            schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_RUN],
        )
        yield get_execution_wrong_schema_mock


@pytest.fixture
def update_execution_mock():
    with patch.object(
        MetadataServiceClient, "update_execution"
    ) as update_execution_mock:
        update_execution_mock.return_value = GapicExecution(
            name=_TEST_EXECUTION_NAME,
            display_name=_TEST_RUN,
            schema_title=constants.SYSTEM_RUN,
            schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_RUN],
            metadata=_TEST_PARAMS,
        )
        yield update_execution_mock


@pytest.fixture
def add_execution_events_mock():
    with patch.object(
        MetadataServiceClient, "add_execution_events"
    ) as add_execution_events_mock:
        add_execution_events_mock.return_value = AddExecutionEventsResponse()
        yield add_execution_events_mock


@pytest.fixture
def list_executions_mock():
    with patch.object(MetadataServiceClient, "list_executions") as list_executions_mock:
        list_executions_mock.return_value = [
            GapicExecution(
                name=_TEST_EXECUTION_NAME,
                display_name=_TEST_RUN,
                schema_title=constants.SYSTEM_RUN,
                schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_RUN],
                metadata=_TEST_PARAMS,
            ),
            GapicExecution(
                name=_TEST_OTHER_EXECUTION_NAME,
                display_name=_TEST_OTHER_RUN,
                schema_title=constants.SYSTEM_RUN,
                schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_RUN],
                metadata=_TEST_OTHER_PARAMS,
            ),
        ]
        yield list_executions_mock


@pytest.fixture
def get_tensorboard_run_not_found_mock():
    with patch.object(
        TensorboardServiceClient, "get_tensorboard_run"
    ) as get_tensorboard_run_mock:
        get_tensorboard_run_mock.side_effect = [
            exceptions.NotFound(""),
            test_constants.TensorboardConstants._TEST_TENSORBOARD_RUN,
        ]
        yield get_tensorboard_run_mock


@pytest.fixture
def get_tensorboard_experiment_not_found_mock():
    with patch.object(
        TensorboardServiceClient, "get_tensorboard_experiment"
    ) as get_tensorboard_experiment_mock:
        get_tensorboard_experiment_mock.side_effect = [
            exceptions.NotFound(""),
            test_constants.TensorboardConstants._TEST_TENSORBOARD_EXPERIMENT,
        ]
        yield get_tensorboard_experiment_mock


@pytest.fixture
def get_tensorboard_time_series_not_found_mock():
    with patch.object(
        TensorboardServiceClient, "get_tensorboard_time_series"
    ) as get_tensorboard_time_series_mock:
        get_tensorboard_time_series_mock.side_effect = [
            exceptions.NotFound(""),
            # test_tensorboard._TEST_TENSORBOARD_TIME_SERIES # change to time series
        ]
        yield get_tensorboard_time_series_mock


@pytest.fixture
def query_execution_inputs_and_outputs_mock():
    with patch.object(
        MetadataServiceClient, "query_execution_inputs_and_outputs"
    ) as query_execution_inputs_and_outputs_mock:
        query_execution_inputs_and_outputs_mock.side_effect = [
            LineageSubgraph(
                artifacts=[
                    GapicArtifact(
                        name=_TEST_ARTIFACT_NAME,
                        display_name=_TEST_ARTIFACT_ID,
                        schema_title=constants.SYSTEM_METRICS,
                        schema_version=constants.SCHEMA_VERSIONS[
                            constants.SYSTEM_METRICS
                        ],
                        metadata=_TEST_METRICS,
                    )
                ],
                events=[
                    gca_event.Event(
                        artifact=_TEST_ARTIFACT_NAME,
                        execution=_TEST_EXECUTION_NAME,
                        type_=gca_event.Event.Type.OUTPUT,
                    )
                ],
            ),
            LineageSubgraph(
                artifacts=[
                    GapicArtifact(
                        name=_TEST_OTHER_ARTIFACT_NAME,
                        display_name=_TEST_OTHER_ARTIFACT_ID,
                        schema_title=constants.SYSTEM_METRICS,
                        schema_version=constants.SCHEMA_VERSIONS[
                            constants.SYSTEM_METRICS
                        ],
                        metadata=_TEST_OTHER_METRICS,
                    ),
                ],
                events=[
                    gca_event.Event(
                        artifact=_TEST_OTHER_ARTIFACT_NAME,
                        execution=_TEST_OTHER_EXECUTION_NAME,
                        type_=gca_event.Event.Type.OUTPUT,
                    )
                ],
            ),
        ]
        yield query_execution_inputs_and_outputs_mock


_TEST_CLASSIFICATION_METRICS_METADATA = {
    "confusionMatrix": {
        "annotationSpecs": [{"displayName": "cat"}, {"displayName": "dog"}],
        "rows": [[9, 1], [1, 9]],
    },
    "confidenceMetrics": [
        {"confidenceThreshold": 0.9, "recall": 0.1, "falsePositiveRate": 0.1},
        {"confidenceThreshold": 0.5, "recall": 0.7, "falsePositiveRate": 0.5},
        {"confidenceThreshold": 0.1, "recall": 0.9, "falsePositiveRate": 0.9},
    ],
}

_TEST_CLASSIFICATION_METRICS_ARTIFACT = GapicArtifact(
    name=_TEST_ARTIFACT_NAME,
    display_name=_TEST_CLASSIFICATION_METRICS["display_name"],
    schema_title=constants.GOOGLE_CLASSIFICATION_METRICS,
    schema_version=constants._DEFAULT_SCHEMA_VERSION,
    metadata=_TEST_CLASSIFICATION_METRICS_METADATA,
    state=GapicArtifact.State.LIVE,
)


@pytest.fixture
def create_classification_metrics_artifact_mock():
    with patch.object(
        MetadataServiceClient, "create_artifact"
    ) as create_classification_metrics_artifact_mock:
        create_classification_metrics_artifact_mock.return_value = (
            _TEST_CLASSIFICATION_METRICS_ARTIFACT
        )
        yield create_classification_metrics_artifact_mock


@pytest.fixture
def get_classification_metrics_artifact_mock():
    with patch.object(
        MetadataServiceClient, "get_artifact"
    ) as get_classification_metrics_artifact_mock:
        get_classification_metrics_artifact_mock.return_value = (
            _TEST_CLASSIFICATION_METRICS_ARTIFACT
        )
        yield get_classification_metrics_artifact_mock


@pytest.fixture
def get_artifact_mock():
    with patch.object(MetadataServiceClient, "get_artifact") as get_artifact_mock:
        get_artifact_mock.return_value = GapicArtifact(
            name=_TEST_ARTIFACT_NAME,
            display_name=_TEST_ARTIFACT_ID,
            schema_title=constants.SYSTEM_METRICS,
            schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_METRICS],
        )
        yield get_artifact_mock


@pytest.fixture
def get_artifact_not_found_mock():
    with patch.object(MetadataServiceClient, "get_artifact") as get_artifact_mock:
        get_artifact_mock.side_effect = exceptions.NotFound("")
        yield get_artifact_mock


@pytest.fixture
def get_artifact_wrong_schema_mock():
    with patch.object(
        MetadataServiceClient, "get_artifact"
    ) as get_artifact_wrong_schema_mock:
        get_artifact_wrong_schema_mock.return_value = GapicArtifact(
            name=_TEST_ARTIFACT_NAME,
            display_name=_TEST_ARTIFACT_ID,
            schema_title=_TEST_WRONG_SCHEMA_TITLE,
            schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_METRICS],
        )
        yield get_artifact_wrong_schema_mock


@pytest.fixture
def update_artifact_mock():
    with patch.object(MetadataServiceClient, "update_artifact") as update_artifact_mock:
        update_artifact_mock.return_value = GapicArtifact(
            name=_TEST_ARTIFACT_NAME,
            display_name=_TEST_ARTIFACT_ID,
            schema_title=constants.SYSTEM_METRICS,
            schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_METRICS],
            metadata=_TEST_METRICS,
        )
        yield update_artifact_mock


def _assert_frame_equal_with_sorted_columns(dataframe_1, dataframe_2):
    try:
        import pandas as pd
    except ImportError:
        raise ImportError(
            "Pandas is not installed and is required to test the get_experiment_df/pipeline_df method. "
            'Please install the SDK using "pip install google-cloud-aiplatform[full]"'
        )

    pd.testing.assert_frame_equal(
        dataframe_1.sort_index(axis=1), dataframe_2.sort_index(axis=1), check_names=True
    )


@pytest.fixture
def mock_storage_blob_upload_from_filename():
    with patch(
        "google.cloud.storage.Blob.upload_from_filename"
    ) as mock_blob_upload_from_filename, patch(
        "google.cloud.storage.Bucket.exists", return_value=True
    ):
        yield mock_blob_upload_from_filename


_TEST_EXPERIMENT_MODEL_ARTIFACT = GapicArtifact(
    name=_TEST_MODEL_NAME,
    display_name=_TEST_DISPLAY_NAME,
    schema_title=constants.GOOGLE_EXPERIMENT_MODEL,
    schema_version=constants._DEFAULT_SCHEMA_VERSION,
    state=GapicArtifact.State.LIVE,
)


@pytest.fixture
def create_experiment_model_artifact_mock():
    with patch.object(
        MetadataServiceClient, "create_artifact"
    ) as create_experiment_model_artifact_mock:
        create_experiment_model_artifact_mock.return_value = (
            _TEST_EXPERIMENT_MODEL_ARTIFACT
        )
        yield create_experiment_model_artifact_mock


@pytest.fixture
def get_experiment_model_artifact_mock():
    with patch.object(
        MetadataServiceClient, "get_artifact"
    ) as get_experiment_model_artifact_mock:
        get_experiment_model_artifact_mock.return_value = (
            _TEST_EXPERIMENT_MODEL_ARTIFACT
        )
        yield get_experiment_model_artifact_mock


@pytest.mark.usefixtures("google_auth_mock")
class TestMetadata:
    def setup_method(self):
        reload(initializer)
        reload(metadata)
        reload(aiplatform)

    def teardown_method(self):
        initializer.global_pool.shutdown(wait=True)

    @pytest.mark.usefixtures("get_pipeline_context_mock")
    def test_get_pipeline_df(
        self, list_executions_mock, query_execution_inputs_and_outputs_mock
    ):
        try:
            import pandas as pd
        except ImportError:
            raise ImportError(
                "Pandas is not installed and is required to test the get_pipeline_df method. "
                'Please install the SDK using "pip install google-cloud-aiplatform[full]"'
            )
        aiplatform.init(project=_TEST_PROJECT, location=_TEST_LOCATION)

        pipeline_df = aiplatform.get_pipeline_df(_TEST_PIPELINE)

        expected_filter = metadata_utils._make_filter_string(
            schema_title=constants.SYSTEM_RUN, in_context=[_TEST_CONTEXT_NAME]
        )

        list_executions_mock.assert_called_once_with(
            request={"parent": _TEST_PARENT, "filter": expected_filter}
        )
        query_execution_inputs_and_outputs_mock.assert_has_calls(
            [
                call(execution=_TEST_EXECUTION_NAME),
                call(execution=_TEST_OTHER_EXECUTION_NAME),
            ]
        )
        pipeline_df_truth = pd.DataFrame(
            [
                {
                    "pipeline_name": _TEST_PIPELINE,
                    "run_name": _TEST_RUN,
                    "param.%s" % _TEST_PARAM_KEY_1: 0.01,
                    "param.%s" % _TEST_PARAM_KEY_2: 0.2,
                    "metric.%s" % _TEST_METRIC_KEY_1: 222,
                    "metric.%s" % _TEST_METRIC_KEY_2: 1,
                },
                {
                    "pipeline_name": _TEST_PIPELINE,
                    "run_name": _TEST_OTHER_RUN,
                    "param.%s" % _TEST_PARAM_KEY_1: 0.02,
                    "param.%s" % _TEST_PARAM_KEY_2: 0.3,
                    "metric.%s" % _TEST_METRIC_KEY_2: 0.9,
                },
            ]
        )

        _assert_frame_equal_with_sorted_columns(pipeline_df, pipeline_df_truth)

    @pytest.mark.usefixtures("get_context_not_found_mock")
    def test_get_pipeline_df_not_exist(self):
        aiplatform.init(project=_TEST_PROJECT, location=_TEST_LOCATION)
        with pytest.raises(exceptions.NotFound):
            aiplatform.get_pipeline_df(_TEST_PIPELINE)

    @pytest.mark.usefixtures("get_context_mock")
    def test_get_pipeline_df_wrong_schema(self):
        aiplatform.init(project=_TEST_PROJECT, location=_TEST_LOCATION)
        with pytest.raises(ValueError):
            aiplatform.get_pipeline_df(_TEST_PIPELINE)


_TEST_EXPERIMENT_RUN_CONTEXT_NAME = f"{_TEST_PARENT}/contexts/{_TEST_EXECUTION_ID}"
_TEST_OTHER_EXPERIMENT_RUN_CONTEXT_NAME = (
    f"{_TEST_PARENT}/contexts/{_TEST_OTHER_EXECUTION_ID}"
)

_EXPERIMENT_MOCK = GapicContext(
    name=_TEST_CONTEXT_NAME,
    display_name=_TEST_EXPERIMENT,
    description=_TEST_EXPERIMENT_DESCRIPTION,
    schema_title=constants.SYSTEM_EXPERIMENT,
    schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_EXPERIMENT],
    metadata={**constants.EXPERIMENT_METADATA},
)

_EXPERIMENT_RUN_MOCK = GapicContext(
    name=_TEST_EXPERIMENT_RUN_CONTEXT_NAME,
    display_name=_TEST_RUN,
    schema_title=constants.SYSTEM_EXPERIMENT_RUN,
    schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_EXPERIMENT_RUN],
    metadata={
        constants._PARAM_KEY: {},
        constants._METRIC_KEY: {},
        constants._STATE_KEY: gca_execution.Execution.State.RUNNING.name,
    },
)

_EXPERIMENT_RUN_MOCK_WITH_PARENT_EXPERIMENT = copy.deepcopy(_EXPERIMENT_RUN_MOCK)
_EXPERIMENT_RUN_MOCK_WITH_PARENT_EXPERIMENT.parent_contexts = [_TEST_CONTEXT_NAME]


@pytest.fixture
def get_experiment_mock():
    with patch.object(MetadataServiceClient, "get_context") as get_context_mock:
        get_context_mock.return_value = _EXPERIMENT_MOCK
        yield get_context_mock


@pytest.fixture
def get_experiment_run_run_mock():
    with patch.object(MetadataServiceClient, "get_context") as get_context_mock:
        get_context_mock.side_effect = [
            _EXPERIMENT_MOCK,
            _EXPERIMENT_RUN_MOCK,
            _EXPERIMENT_RUN_MOCK_WITH_PARENT_EXPERIMENT,
        ]

        yield get_context_mock


@pytest.fixture
def get_experiment_run_mock():
    with patch.object(MetadataServiceClient, "get_context") as get_context_mock:
        get_context_mock.side_effect = [
            _EXPERIMENT_MOCK,
            _EXPERIMENT_RUN_MOCK_WITH_PARENT_EXPERIMENT,
        ]

        yield get_context_mock


@pytest.fixture
def create_experiment_context_mock():
    with patch.object(MetadataServiceClient, "create_context") as create_context_mock:
        create_context_mock.side_effect = [_TEST_EXPERIMENT_CONTEXT]
        yield create_context_mock


@pytest.fixture
def create_experiment_run_context_mock():
    with patch.object(MetadataServiceClient, "create_context") as create_context_mock:
        create_context_mock.side_effect = [_EXPERIMENT_RUN_MOCK]
        yield create_context_mock


@pytest.fixture
def update_experiment_run_context_to_running():
    with patch.object(MetadataServiceClient, "update_context") as update_context_mock:
        update_context_mock.side_effect = [_EXPERIMENT_RUN_MOCK]
        yield update_context_mock


@pytest.fixture
def create_execution_mock():
    with patch.object(
        MetadataServiceClient, "create_execution"
    ) as create_execution_mock:
        create_execution_mock.side_effect = [_TEST_EXECUTION]
        yield create_execution_mock


@pytest.fixture
def update_context_mock_v2():
    with patch.object(MetadataServiceClient, "update_context") as update_context_mock:
        update_context_mock.side_effect = [
            # experiment run
            GapicContext(
                name=_TEST_EXPERIMENT_RUN_CONTEXT_NAME,
                display_name=_TEST_RUN,
                schema_title=constants.SYSTEM_EXPERIMENT_RUN,
                schema_version=constants.SCHEMA_VERSIONS[
                    constants.SYSTEM_EXPERIMENT_RUN
                ],
                metadata={**constants.EXPERIMENT_METADATA},
            ),
            # experiment run
            GapicContext(
                name=_TEST_EXPERIMENT_RUN_CONTEXT_NAME,
                display_name=_TEST_RUN,
                schema_title=constants.SYSTEM_EXPERIMENT_RUN,
                schema_version=constants.SCHEMA_VERSIONS[
                    constants.SYSTEM_EXPERIMENT_RUN
                ],
                metadata=constants.EXPERIMENT_METADATA,
                parent_contexts=[_TEST_CONTEXT_NAME],
            ),
        ]

        yield update_context_mock


@pytest.fixture
def list_contexts_mock():
    with patch.object(MetadataServiceClient, "list_contexts") as list_contexts_mock:
        list_contexts_mock.return_value = [
            GapicContext(
                name=_TEST_EXPERIMENT_RUN_CONTEXT_NAME,
                display_name=_TEST_RUN,
                schema_title=constants.SYSTEM_EXPERIMENT_RUN,
                schema_version=constants.SCHEMA_VERSIONS[
                    constants.SYSTEM_EXPERIMENT_RUN
                ],
                metadata=constants.EXPERIMENT_METADATA,
                parent_contexts=[_TEST_CONTEXT_NAME],
            ),
            GapicContext(
                name=_TEST_OTHER_EXPERIMENT_RUN_CONTEXT_NAME,
                display_name=_TEST_OTHER_RUN,
                schema_title=constants.SYSTEM_EXPERIMENT_RUN,
                schema_version=constants.SCHEMA_VERSIONS[
                    constants.SYSTEM_EXPERIMENT_RUN
                ],
                metadata=constants.EXPERIMENT_METADATA,
                parent_contexts=[_TEST_CONTEXT_NAME],
            ),
        ]
        yield list_contexts_mock


@pytest.fixture
def add_context_children_mock():
    with patch.object(
        MetadataServiceClient, "add_context_children"
    ) as add_context_children_mock:
        yield add_context_children_mock


_EXPERIMENT_RUN_MOCK_POPULATED_1 = copy.deepcopy(
    _EXPERIMENT_RUN_MOCK_WITH_PARENT_EXPERIMENT
)
_EXPERIMENT_RUN_MOCK_POPULATED_1.metadata[constants._PARAM_KEY].update(_TEST_PARAMS)
_EXPERIMENT_RUN_MOCK_POPULATED_1.metadata[constants._METRIC_KEY].update(_TEST_METRICS)
_EXPERIMENT_RUN_MOCK_POPULATED_2 = copy.deepcopy(
    _EXPERIMENT_RUN_MOCK_WITH_PARENT_EXPERIMENT
)
_EXPERIMENT_RUN_MOCK_POPULATED_2.display_name = _TEST_OTHER_RUN
_EXPERIMENT_RUN_MOCK_POPULATED_2.metadata[constants._PARAM_KEY].update(
    _TEST_OTHER_PARAMS
)
_EXPERIMENT_RUN_MOCK_POPULATED_2.metadata[constants._METRIC_KEY].update(
    _TEST_OTHER_METRICS
)

_TEST_PIPELINE_RUN_ID = "test-pipeline-run"
_TEST_PIPELINE_RUN_CONTEXT_NAME = f"{_TEST_PARENT}/contexts/{_TEST_PIPELINE_RUN_ID}"

_TEST_PIPELINE_CONTEXT = GapicContext(
    name=_TEST_PIPELINE_RUN_CONTEXT_NAME,
    display_name=_TEST_PIPELINE_RUN_ID,
    schema_title=constants.SYSTEM_PIPELINE_RUN,
    parent_contexts=[_TEST_CONTEXT_NAME],
)


@pytest.fixture()
def list_context_mock_for_experiment_dataframe_mock():
    with patch.object(MetadataServiceClient, "list_contexts") as list_context_mock:
        list_context_mock.side_effect = [
            # experiment runs
            [
                _EXPERIMENT_RUN_MOCK_POPULATED_1,
                _EXPERIMENT_RUN_MOCK_POPULATED_2,
                _TEST_PIPELINE_CONTEXT,
            ],
            # pipeline runs
            [],
        ]
        yield list_context_mock


_TEST_LEGACY_METRIC_ARTIFACT = GapicArtifact(
    name=_TEST_ARTIFACT_NAME,
    schema_title=constants.SYSTEM_METRICS,
    metadata=_TEST_METRICS,
)

_TEST_PIPELINE_METRIC_ARTIFACT = GapicArtifact(
    name=_TEST_ARTIFACT_NAME,
    schema_title=constants.SYSTEM_METRICS,
    metadata={key: value + 1 for key, value in _TEST_METRICS.items()},
)


@pytest.fixture()
def list_artifact_mock_for_experiment_dataframe():
    with patch.object(MetadataServiceClient, "list_artifacts") as list_artifacts_mock:
        list_artifacts_mock.side_effect = [
            # pipeline run metric artifact
            [_TEST_PIPELINE_METRIC_ARTIFACT],
        ]
        yield list_artifacts_mock


_TEST_PIPELINE_SYSTEM_RUN_EXECUTION = GapicExecution(
    name=_TEST_EXECUTION_NAME,
    schema_title=constants.SYSTEM_RUN,
    state=gca_execution.Execution.State.RUNNING,
    metadata={f"input:{key}": value + 1 for key, value in _TEST_PARAMS.items()},
)

_TEST_LEGACY_SYSTEM_RUN_EXECUTION = GapicExecution(
    name=_TEST_EXECUTION_NAME,
    display_name=_TEST_RUN,
    schema_title=constants.SYSTEM_RUN,
    schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_RUN],
    metadata=_TEST_PARAMS,
)


# backward compatibility
@pytest.fixture()
def list_executions_mock_for_experiment_dataframe():
    with patch.object(MetadataServiceClient, "list_executions") as list_executions_mock:
        list_executions_mock.side_effect = [
            # legacy system.run execution
            [_TEST_LEGACY_SYSTEM_RUN_EXECUTION],
            # pipeline system.run execution
            [_TEST_PIPELINE_SYSTEM_RUN_EXECUTION],
        ]
        yield list_executions_mock


@pytest.fixture
def get_tensorboard_run_artifact_not_found_mock():
    with patch.object(MetadataServiceClient, "get_artifact") as get_artifact_mock:
        get_artifact_mock.side_effect = exceptions.NotFound("")
        yield get_artifact_mock


_TEST_LEGACY_METRIC_ARTIFACT

_TEST_TENSORBOARD_RUN_ARTIFACT = GapicArtifact(
    name=experiment_run_resource.ExperimentRun._tensorboard_run_id(
        _TEST_EXPERIMENT_RUN_CONTEXT_NAME
    ),
    uri="https://us-central1-aiplatform.googleapis.com/v1/projects/test-project/locations/us-central1/tensorboards/1028944691210842416/experiments/test-experiment/runs/test-run",
    schema_title=google.cloud.aiplatform.metadata.constants._TENSORBOARD_RUN_REFERENCE_ARTIFACT.schema_title,
    schema_version=google.cloud.aiplatform.metadata.constants._TENSORBOARD_RUN_REFERENCE_ARTIFACT.schema_version,
    state=GapicArtifact.State.LIVE,
    metadata={
        google.cloud.aiplatform.metadata.constants._VERTEX_EXPERIMENT_TRACKING_LABEL: True,
        constants.GCP_ARTIFACT_RESOURCE_NAME_KEY: test_constants.TensorboardConstants._TEST_TENSORBOARD_RUN_NAME,
    },
)


@pytest.fixture
def list_tensorboard_time_series_mock_empty():
    with patch.object(
        TensorboardServiceClient,
        "list_tensorboard_time_series",
    ) as list_tensorboard_time_series_mock:
        list_tensorboard_time_series_mock.side_effect = [
            [],  # initially empty
            [],
            [test_constants.TensorboardConstants._TEST_TENSORBOARD_TIME_SERIES],
        ]
        yield list_tensorboard_time_series_mock


@pytest.fixture
def create_tensorboard_run_artifact_mock():
    with patch.object(MetadataServiceClient, "create_artifact") as create_artifact_mock:
        create_artifact_mock.side_effect = [_TEST_TENSORBOARD_RUN_ARTIFACT]
        yield create_artifact_mock


@pytest.fixture
def get_tensorboard_run_artifact_mock():
    with patch.object(MetadataServiceClient, "get_artifact") as get_artifact_mock:
        get_artifact_mock.side_effect = [
            _TEST_TENSORBOARD_RUN_ARTIFACT,
            exceptions.NotFound(""),
            _TEST_LEGACY_METRIC_ARTIFACT,
        ]
        yield get_artifact_mock


@pytest.mark.usefixtures("google_auth_mock")
class TestExperiments:
    def setup_method(self):
        reload(initializer)
        reload(metadata)
        reload(aiplatform)

    def teardown_method(self):
        initializer.global_pool.shutdown(wait=True)

    def test_init_experiment_with_existing_metadataStore_and_context(
        self, get_metadata_store_mock, get_experiment_run_run_mock
    ):
        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            experiment=_TEST_EXPERIMENT,
        )

        get_metadata_store_mock.assert_called_once_with(
            name=_TEST_METADATASTORE, retry=base._DEFAULT_RETRY
        )
        get_experiment_run_run_mock.assert_called_once_with(
            name=_TEST_CONTEXT_NAME, retry=base._DEFAULT_RETRY
        )

    @pytest.mark.usefixtures("get_metadata_store_mock")
    def test_create_experiment(self, create_experiment_context_mock):
        exp = aiplatform.Experiment.create(
            experiment_name=_TEST_EXPERIMENT,
            description=_TEST_EXPERIMENT_DESCRIPTION,
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
        )

        _TRUE_CONTEXT = copy.deepcopy(_TEST_EXPERIMENT_CONTEXT)
        _TRUE_CONTEXT.name = None
        _TRUE_CONTEXT.metadata.pop("backing_tensorboard_resource")

        create_experiment_context_mock.assert_called_once_with(
            parent=_TEST_PARENT, context=_TRUE_CONTEXT, context_id=_TEST_EXPERIMENT
        )

        assert exp._metadata_context.gca_resource == _TEST_EXPERIMENT_CONTEXT

    def test_init_experiment_with_credentials(
        self,
        get_metadata_store_mock,
        get_experiment_run_run_mock,
    ):
        creds = credentials.AnonymousCredentials()

        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            experiment=_TEST_EXPERIMENT,
            credentials=creds,
        )

        assert (
            metadata._experiment_tracker._experiment._metadata_context.api_client._transport._credentials
            == creds
        )

        get_metadata_store_mock.assert_called_once_with(
            name=_TEST_METADATASTORE, retry=base._DEFAULT_RETRY
        )
        get_experiment_run_run_mock.assert_called_once_with(
            name=_TEST_CONTEXT_NAME, retry=base._DEFAULT_RETRY
        )

    def test_init_and_get_metadata_store_with_credentials(
        self, get_metadata_store_mock
    ):
        creds = credentials.AnonymousCredentials()

        aiplatform.init(
            project=_TEST_PROJECT, location=_TEST_LOCATION, credentials=creds
        )

        store = metadata_store._MetadataStore.get_or_create()

        assert store.api_client._transport._credentials == creds

    @pytest.mark.usefixtures(
        "get_metadata_store_mock_raise_not_found_exception",
        "create_metadata_store_mock",
    )
    def test_init_and_get_then_create_metadata_store_with_credentials(
        self,
    ):
        creds = credentials.AnonymousCredentials()

        aiplatform.init(
            project=_TEST_PROJECT, location=_TEST_LOCATION, credentials=creds
        )

        store = metadata_store._MetadataStore.get_or_create()

        assert store.api_client._transport._credentials == creds

    def test_init_experiment_with_existing_description(
        self, get_metadata_store_mock, get_experiment_run_run_mock
    ):
        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            experiment=_TEST_EXPERIMENT,
            experiment_description=_TEST_EXPERIMENT_DESCRIPTION,
        )

        get_metadata_store_mock.assert_called_once_with(
            name=_TEST_METADATASTORE, retry=base._DEFAULT_RETRY
        )
        get_experiment_run_run_mock.assert_called_once_with(
            name=_TEST_CONTEXT_NAME, retry=base._DEFAULT_RETRY
        )

    @pytest.mark.usefixtures("get_metadata_store_mock", "get_experiment_run_run_mock")
    def test_init_experiment_without_existing_description(
        self,
        update_context_mock,
    ):
        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            experiment=_TEST_EXPERIMENT,
            experiment_description=_TEST_OTHER_EXPERIMENT_DESCRIPTION,
        )

        experiment_context = GapicContext(
            name=_TEST_CONTEXT_NAME,
            display_name=_TEST_EXPERIMENT,
            description=_TEST_OTHER_EXPERIMENT_DESCRIPTION,
            schema_title=constants.SYSTEM_EXPERIMENT,
            schema_version=constants.SCHEMA_VERSIONS[constants.SYSTEM_EXPERIMENT],
            metadata=constants.EXPERIMENT_METADATA,
        )

        update_context_mock.assert_called_once_with(context=experiment_context)

    @pytest.mark.usefixtures(
        "get_metadata_store_mock",
        "get_experiment_run_mock",
        "update_experiment_run_context_to_running",
        "get_tensorboard_run_artifact_not_found_mock",
    )
    def test_init_experiment_reset(self):
        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            experiment=_TEST_EXPERIMENT,
        )
        aiplatform.start_run(_TEST_RUN, resume=True)

        aiplatform.init(project=_TEST_PROJECT, location=_TEST_LOCATION)

        assert metadata._experiment_tracker.experiment_name == _TEST_EXPERIMENT
        assert metadata._experiment_tracker.experiment_run.name == _TEST_RUN

        aiplatform.init(project=_TEST_OTHER_PROJECT, location=_TEST_LOCATION)

        assert metadata._experiment_tracker.experiment_name is None
        assert metadata._experiment_tracker.experiment_run is None

    @pytest.mark.usefixtures("get_metadata_store_mock", "get_context_wrong_schema_mock")
    def test_init_experiment_wrong_schema(self):
        with pytest.raises(ValueError):
            aiplatform.init(
                project=_TEST_PROJECT,
                location=_TEST_LOCATION,
                experiment=_TEST_EXPERIMENT,
            )

    @pytest.mark.usefixtures("get_metadata_store_mock")
    def test_start_run(
        self,
        get_experiment_mock,
        create_experiment_run_context_mock,
        add_context_children_mock,
    ):

        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            experiment=_TEST_EXPERIMENT,
        )
        aiplatform.start_run(_TEST_RUN)

        get_experiment_mock.assert_called_with(
            name=_TEST_CONTEXT_NAME, retry=base._DEFAULT_RETRY
        )

        _TRUE_CONTEXT = copy.deepcopy(_EXPERIMENT_RUN_MOCK)
        _TRUE_CONTEXT.name = None

        create_experiment_run_context_mock.assert_called_with(
            parent=_TEST_METADATASTORE,
            context=_TRUE_CONTEXT,
            context_id=_EXPERIMENT_RUN_MOCK.name.split("/")[-1],
        )

        add_context_children_mock.assert_called_with(
            context=_EXPERIMENT_MOCK.name, child_contexts=[_EXPERIMENT_RUN_MOCK.name]
        )

    @pytest.mark.usefixtures("get_metadata_store_mock", "get_experiment_mock")
    def test_start_run_fails_when_run_name_too_long(self):

        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            experiment=_TEST_EXPERIMENT,
        )

        run_name_too_long = "".join(
            "a"
            for _ in range(
                constants._EXPERIMENT_RUN_MAX_LENGTH + 2 - len(_TEST_EXPERIMENT)
            )
        )

        with pytest.raises(ValueError):
            aiplatform.start_run(run_name_too_long)

    @pytest.mark.usefixtures(
        "get_metadata_store_mock",
        "get_experiment_mock",
        "create_experiment_run_context_mock",
        "add_context_children_mock",
    )
    def test_log_params(
        self,
        update_context_mock,
    ):
        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            experiment=_TEST_EXPERIMENT,
        )
        aiplatform.start_run(_TEST_RUN)
        aiplatform.log_params(_TEST_PARAMS)

        _TRUE_CONTEXT = copy.deepcopy(_EXPERIMENT_RUN_MOCK)
        _TRUE_CONTEXT.metadata[constants._PARAM_KEY].update(_TEST_PARAMS)

        update_context_mock.assert_called_once_with(context=_TRUE_CONTEXT)

    @pytest.mark.usefixtures(
        "get_metadata_store_mock",
        "get_experiment_mock",
        "create_experiment_run_context_mock",
        "add_context_children_mock",
    )
    def test_log_metrics(self, update_context_mock):
        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            experiment=_TEST_EXPERIMENT,
        )
        aiplatform.start_run(_TEST_RUN)
        aiplatform.log_metrics(_TEST_METRICS)

        _TRUE_CONTEXT = copy.deepcopy(_EXPERIMENT_RUN_MOCK)
        _TRUE_CONTEXT.metadata[constants._METRIC_KEY].update(_TEST_METRICS)

        update_context_mock.assert_called_once_with(context=_TRUE_CONTEXT)

    @pytest.mark.usefixtures(
        "get_metadata_store_mock",
        "get_experiment_mock",
        "create_experiment_run_context_mock",
        "add_context_children_mock",
    )
    def test_log_classification_metrics(
        self,
        create_classification_metrics_artifact_mock,
        get_classification_metrics_artifact_mock,
        add_context_artifacts_and_executions_mock,
    ):
        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            experiment=_TEST_EXPERIMENT,
        )
        aiplatform.start_run(_TEST_RUN)
        classification_metrics = aiplatform.log_classification_metrics(
            display_name=_TEST_CLASSIFICATION_METRICS["display_name"],
            labels=_TEST_CLASSIFICATION_METRICS["labels"],
            matrix=_TEST_CLASSIFICATION_METRICS["matrix"],
            fpr=_TEST_CLASSIFICATION_METRICS["fpr"],
            tpr=_TEST_CLASSIFICATION_METRICS["tpr"],
            threshold=_TEST_CLASSIFICATION_METRICS["threshold"],
        )

        expected_artifact = GapicArtifact(
            display_name=_TEST_CLASSIFICATION_METRICS["display_name"],
            schema_title=constants.GOOGLE_CLASSIFICATION_METRICS,
            schema_version=constants._DEFAULT_SCHEMA_VERSION,
            metadata=_TEST_CLASSIFICATION_METRICS_METADATA,
            state=GapicArtifact.State.LIVE,
        )
        create_classification_metrics_artifact_mock.assert_called_once_with(
            parent=_TEST_PARENT,
            artifact=expected_artifact,
            artifact_id=None,
        )

        get_classification_metrics_artifact_mock.assert_called_once_with(
            name=_TEST_ARTIFACT_NAME, retry=base._DEFAULT_RETRY
        )
        assert isinstance(
            classification_metrics, google_artifact_schema.ClassificationMetrics
        )

        add_context_artifacts_and_executions_mock.assert_called_once_with(
            context=_TEST_EXPERIMENT_RUN_CONTEXT_NAME,
            artifacts=[_TEST_ARTIFACT_NAME],
            executions=None,
        )

    @pytest.mark.usefixtures(
        "get_metadata_store_mock",
        "get_experiment_mock",
        "create_experiment_run_context_mock",
        "add_context_children_mock",
        "mock_storage_blob_upload_from_filename",
        "create_experiment_model_artifact_mock",
        "get_experiment_model_artifact_mock",
        "get_metadata_store_mock",
    )
    def test_log_model(
        self,
        add_context_artifacts_and_executions_mock,
    ):
        train_x = np.array([[1, 1], [1, 2], [2, 2], [2, 3]])
        train_y = np.dot(train_x, np.array([1, 2])) + 3
        model = LinearRegression()
        model.fit(train_x, train_y)

        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            staging_bucket=_TEST_BUCKET_NAME,
            credentials=_TEST_CREDENTIALS,
            experiment=_TEST_EXPERIMENT,
        )
        aiplatform.start_run(_TEST_RUN)
        aiplatform.log_model(model, _TEST_MODEL_ID)

        add_context_artifacts_and_executions_mock.assert_called_once_with(
            context=_TEST_EXPERIMENT_RUN_CONTEXT_NAME,
            artifacts=[_TEST_MODEL_NAME],
            executions=None,
        )

    @pytest.mark.usefixtures(
        "get_metadata_store_mock",
        "get_experiment_mock",
        "create_experiment_run_context_mock",
        "add_context_children_mock",
        "get_tensorboard_mock",
        "get_tensorboard_run_not_found_mock",
        "get_tensorboard_experiment_not_found_mock",
        "get_artifact_not_found_mock",
        "get_tensorboard_time_series_not_found_mock",
        "list_tensorboard_time_series_mock_empty",
    )
    def test_log_time_series_metrics(
        self,
        update_context_mock,
        create_tensorboard_experiment_mock,
        create_tensorboard_run_mock,
        create_tensorboard_run_artifact_mock,
        add_context_artifacts_and_executions_mock,
        create_tensorboard_time_series_mock,
        batch_read_tensorboard_time_series_mock,
        write_tensorboard_run_data_mock,
    ):
        tb = aiplatform.Tensorboard(
            test_constants.TensorboardConstants._TEST_TENSORBOARD_NAME
        )

        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            experiment=_TEST_EXPERIMENT,
            experiment_tensorboard=tb,
        )

        update_context_mock.assert_called_once_with(context=_TEST_EXPERIMENT_CONTEXT)

        aiplatform.start_run(_TEST_RUN)
        timestamp = utils.get_timestamp_proto()
        aiplatform.log_time_series_metrics(_TEST_OTHER_METRICS, wall_time=timestamp)

        create_tensorboard_experiment_mock.assert_called_once_with(
            parent=test_constants.TensorboardConstants._TEST_TENSORBOARD_NAME,
            tensorboard_experiment_id=_TEST_CONTEXT_ID,
            tensorboard_experiment=gca_tensorboard_experiment.TensorboardExperiment(
                display_name=experiment_run_resource.ExperimentRun._format_tensorboard_experiment_display_name(
                    _TEST_CONTEXT_ID
                ),
                labels=constants._VERTEX_EXPERIMENT_TB_EXPERIMENT_LABEL,
            ),
            metadata=(),
            timeout=None,
        )

        create_tensorboard_run_mock.assert_called_once_with(
            parent=test_constants.TensorboardConstants._TEST_TENSORBOARD_EXPERIMENT_NAME,
            tensorboard_run_id=_TEST_RUN,
            tensorboard_run=gca_tensorboard_run.TensorboardRun(
                display_name=_TEST_RUN,
            ),
            metadata=(),
            timeout=None,
        )

        true_tb_run_artifact = copy.deepcopy(_TEST_TENSORBOARD_RUN_ARTIFACT)
        true_tb_run_artifact.name = None

        create_tensorboard_run_artifact_mock.assert_called_once_with(
            parent=_TEST_PARENT,
            artifact=true_tb_run_artifact,
            artifact_id=experiment_run_resource.ExperimentRun._tensorboard_run_id(
                _TEST_EXECUTION_ID
            ),
        )

        add_context_artifacts_and_executions_mock.assert_called_once_with(
            context=_TEST_EXPERIMENT_RUN_CONTEXT_NAME,
            artifacts=[_TEST_TENSORBOARD_RUN_ARTIFACT.name],
            executions=None,
        )

        create_tensorboard_time_series_mock.assert_called_with(
            parent=test_constants.TensorboardConstants._TEST_TENSORBOARD_RUN_NAME,
            tensorboard_time_series=gca_tensorboard_time_series.TensorboardTimeSeries(
                display_name=list(_TEST_OTHER_METRICS.keys())[0],
                value_type="SCALAR",
                plugin_name="scalars",
            ),
        )

        ts_data = [
            gca_tensorboard_data.TimeSeriesData(
                tensorboard_time_series_id=test_constants.TensorboardConstants._TEST_TENSORBOARD_TIME_SERIES_ID,
                value_type=gca_tensorboard_time_series.TensorboardTimeSeries.ValueType.SCALAR,
                values=[
                    gca_tensorboard_data.TimeSeriesDataPoint(
                        scalar=gca_tensorboard_data.Scalar(value=value),
                        wall_time=timestamp,
                        step=2,
                    )
                ],
            )
            for value in _TEST_OTHER_METRICS.values()
        ]

        write_tensorboard_run_data_mock.assert_called_once_with(
            tensorboard_run=test_constants.TensorboardConstants._TEST_TENSORBOARD_RUN_NAME,
            time_series_data=ts_data,
        )

    @pytest.mark.usefixtures(
        "get_metadata_store_mock",
        "get_experiment_mock",
        "create_experiment_run_context_mock",
        "add_context_children_mock",
    )
    def test_log_metrics_nest_value_raises_error(self):
        aiplatform.init(
            project=_TEST_PROJECT, location=_TEST_LOCATION, experiment=_TEST_EXPERIMENT
        )
        aiplatform.start_run(_TEST_RUN)
        with pytest.raises(TypeError):
            aiplatform.log_metrics({"test": {"nested": "string"}})

    @pytest.mark.usefixtures(
        "get_metadata_store_mock",
        "get_experiment_mock",
        "create_experiment_run_context_mock",
        "add_context_children_mock",
    )
    def test_log_params_nest_value_raises_error(self):
        aiplatform.init(
            project=_TEST_PROJECT, location=_TEST_LOCATION, experiment=_TEST_EXPERIMENT
        )
        aiplatform.start_run(_TEST_RUN)
        with pytest.raises(TypeError):
            aiplatform.log_params({"test": {"nested": "string"}})

    @pytest.mark.usefixtures(
        "get_metadata_store_mock",
        "get_experiment_mock",
        "create_experiment_run_context_mock",
        "add_context_children_mock",
        "get_artifact_mock",
    )
    def test_start_execution_and_assign_artifact(
        self,
        create_execution_mock,
        add_execution_events_mock,
        add_context_artifacts_and_executions_mock,
        update_execution_mock,
    ):
        aiplatform.init(
            project=_TEST_PROJECT, location=_TEST_LOCATION, experiment=_TEST_EXPERIMENT
        )
        aiplatform.start_run(_TEST_RUN)

        in_artifact = aiplatform.Artifact(_TEST_ARTIFACT_ID)
        out_artifact = aiplatform.Artifact(_TEST_ARTIFACT_ID)

        with aiplatform.start_execution(
            schema_title=_TEST_SCHEMA_TITLE,
            display_name=_TEST_DISPLAY_NAME,
            metadata=_TEST_METADATA,
        ) as exc:

            exc.assign_input_artifacts([in_artifact])
            exc.assign_output_artifacts([out_artifact])

        _created_execution = copy.deepcopy(_TEST_EXECUTION)
        _created_execution.name = None

        create_execution_mock.assert_called_once_with(
            parent=_TEST_PARENT, execution=_created_execution, execution_id=None
        )

        in_event = gca_event.Event(
            artifact=_TEST_ARTIFACT_NAME,
            type_=gca_event.Event.Type.INPUT,
        )

        out_event = gca_event.Event(
            artifact=_TEST_ARTIFACT_NAME,
            type_=gca_event.Event.Type.OUTPUT,
        )

        add_execution_events_mock.assert_has_calls(
            [
                call(execution=_TEST_EXECUTION.name, events=[in_event]),
                call(execution=_TEST_EXECUTION.name, events=[out_event]),
            ]
        )

        add_context_artifacts_and_executions_mock.assert_has_calls(
            [
                call(
                    context=_TEST_EXPERIMENT_RUN_CONTEXT_NAME,
                    artifacts=None,
                    executions=[_TEST_EXECUTION.name],
                ),
                call(
                    context=_TEST_EXPERIMENT_RUN_CONTEXT_NAME,
                    artifacts=[_TEST_ARTIFACT_NAME],
                    executions=[],
                ),
                call(
                    context=_TEST_EXPERIMENT_RUN_CONTEXT_NAME,
                    artifacts=[_TEST_ARTIFACT_NAME],
                    executions=[],
                ),
            ]
        )

        updated_execution = copy.deepcopy(_TEST_EXECUTION)
        updated_execution.state = GapicExecution.State.COMPLETE

        update_execution_mock.assert_called_once_with(execution=updated_execution)

    @pytest.mark.usefixtures(
        "get_metadata_store_mock",
        "get_experiment_mock",
        "create_experiment_run_context_mock",
        "add_context_children_mock",
    )
    def test_end_run(
        self,
        update_context_mock,
    ):
        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            experiment=_TEST_EXPERIMENT,
        )
        aiplatform.start_run(_TEST_RUN)
        aiplatform.end_run()

        _TRUE_CONTEXT = copy.deepcopy(_EXPERIMENT_RUN_MOCK)
        _TRUE_CONTEXT.metadata[
            constants._STATE_KEY
        ] = gca_execution.Execution.State.COMPLETE.name

        update_context_mock.assert_called_once_with(context=_TRUE_CONTEXT)

    @pytest.mark.usefixtures(
        "get_metadata_store_mock",
        "get_experiment_mock",
        "create_experiment_run_context_mock",
        "get_pipeline_job_mock",
    )
    def test_log_pipeline_job(
        self,
        add_context_children_mock,
    ):
        aiplatform.init(
            project=_TEST_PROJECT,
            location=_TEST_LOCATION,
            experiment=_TEST_EXPERIMENT,
        )
        aiplatform.start_run(_TEST_RUN)

        pipeline_job = aiplatform.PipelineJob.get(
            test_constants.PipelineJobConstants._TEST_PIPELINE_JOB_ID
        )
        pipeline_job.wait()

        aiplatform.log(pipeline_job=pipeline_job)

        add_context_children_mock.assert_called_with(
            context=_EXPERIMENT_RUN_MOCK.name,
            child_contexts=[
                pipeline_job.gca_resource.job_detail.pipeline_run_context.name
            ],
        )

    @pytest.mark.usefixtures(
        "get_experiment_mock",
        "list_tensorboard_time_series_mock",
        "batch_read_tensorboard_time_series_mock",
    )
    def test_get_experiment_df(
        self,
        list_context_mock_for_experiment_dataframe_mock,
        list_artifact_mock_for_experiment_dataframe,
        list_executions_mock_for_experiment_dataframe,
        get_tensorboard_run_artifact_mock,
        get_tensorboard_run_mock,
    ):
        import pandas as pd

        aiplatform.init(project=_TEST_PROJECT, location=_TEST_LOCATION)

        experiment_df = aiplatform.get_experiment_df(_TEST_EXPERIMENT)

        expected_filter = metadata_utils._make_filter_string(
            parent_contexts=[_TEST_CONTEXT_NAME],
            schema_title=[
                constants.SYSTEM_EXPERIMENT_RUN,
                constants.SYSTEM_PIPELINE_RUN,
            ],
        )

        list_context_mock_for_experiment_dataframe_mock.assert_called_once_with(
            request=dict(parent=_TEST_PARENT, filter=expected_filter)
        )

        expected_legacy_filter = metadata_utils._make_filter_string(
            in_context=[_TEST_CONTEXT_NAME], schema_title=[constants.SYSTEM_RUN]
        )
        expected_pipeline_filter = metadata_utils._make_filter_string(
            in_context=[_TEST_PIPELINE_CONTEXT.name], schema_title=constants.SYSTEM_RUN
        )

        list_executions_mock_for_experiment_dataframe.assert_has_calls(
            calls=[
                call(request=dict(parent=_TEST_PARENT, filter=expected_legacy_filter)),
                call(
                    request=dict(parent=_TEST_PARENT, filter=expected_pipeline_filter)
                ),
            ],
            any_order=False,
        )

        expected_filter = metadata_utils._make_filter_string(
            in_context=[_TEST_PIPELINE_CONTEXT.name],
            schema_title=[
                constants.SYSTEM_METRICS,
                constants.GOOGLE_CLASSIFICATION_METRICS,
                constants.GOOGLE_REGRESSION_METRICS,
                constants.GOOGLE_FORECASTING_METRICS,
            ],
        )

        list_artifact_mock_for_experiment_dataframe.assert_has_calls(
            calls=[call(request=dict(parent=_TEST_PARENT, filter=expected_filter))],
            any_order=False,
        )

        experiment_df_truth = pd.DataFrame(
            [
                {
                    "experiment_name": _TEST_EXPERIMENT,
                    "run_type": constants.SYSTEM_EXPERIMENT_RUN,
                    "state": gca_execution.Execution.State.RUNNING.name,
                    "run_name": _TEST_RUN,
                    "param.%s" % _TEST_PARAM_KEY_1: _TEST_PARAMS[_TEST_PARAM_KEY_1],
                    "param.%s" % _TEST_PARAM_KEY_2: _TEST_PARAMS[_TEST_PARAM_KEY_2],
                    "metric.%s" % _TEST_METRIC_KEY_1: _TEST_METRICS[_TEST_METRIC_KEY_1],
                    "metric.%s" % _TEST_METRIC_KEY_2: _TEST_METRICS[_TEST_METRIC_KEY_2],
                    "time_series_metric.accuracy": test_constants.TensorboardConstants._TEST_TENSORBOARD_TIME_SERIES_DATA.values[
                        0
                    ].scalar.value,
                },
                {
                    "experiment_name": _TEST_EXPERIMENT,
                    "run_type": constants.SYSTEM_EXPERIMENT_RUN,
                    "state": gca_execution.Execution.State.RUNNING.name,
                    "run_name": _TEST_OTHER_RUN,
                    "param.%s"
                    % _TEST_PARAM_KEY_1: _TEST_OTHER_PARAMS[_TEST_PARAM_KEY_1],
                    "param.%s"
                    % _TEST_PARAM_KEY_2: _TEST_OTHER_PARAMS[_TEST_PARAM_KEY_2],
                    "metric.%s"
                    % _TEST_METRIC_KEY_2: _TEST_OTHER_METRICS[_TEST_METRIC_KEY_2],
                },
                {
                    "experiment_name": _TEST_EXPERIMENT,
                    "run_type": constants.SYSTEM_PIPELINE_RUN,
                    "state": gca_execution.Execution.State.RUNNING.name,
                    "run_name": _TEST_PIPELINE_RUN_ID,
                    "param.%s" % _TEST_PARAM_KEY_1: _TEST_PARAMS[_TEST_PARAM_KEY_1] + 1,
                    "param.%s" % _TEST_PARAM_KEY_2: _TEST_PARAMS[_TEST_PARAM_KEY_2] + 1,
                    "metric.%s" % _TEST_METRIC_KEY_1: _TEST_METRICS[_TEST_METRIC_KEY_1]
                    + 1,
                    "metric.%s" % _TEST_METRIC_KEY_2: _TEST_METRICS[_TEST_METRIC_KEY_2]
                    + 1,
                },
                {
                    "experiment_name": _TEST_EXPERIMENT,
                    "run_type": constants.SYSTEM_RUN,
                    "state": gca_execution.Execution.State.STATE_UNSPECIFIED.name,
                    "run_name": _TEST_RUN,
                    "param.%s" % _TEST_PARAM_KEY_1: _TEST_PARAMS[_TEST_PARAM_KEY_1],
                    "param.%s" % _TEST_PARAM_KEY_2: _TEST_PARAMS[_TEST_PARAM_KEY_2],
                    "metric.%s" % _TEST_METRIC_KEY_1: _TEST_METRICS[_TEST_METRIC_KEY_1],
                    "metric.%s" % _TEST_METRIC_KEY_2: _TEST_METRICS[_TEST_METRIC_KEY_2],
                },
            ]
        )

        _assert_frame_equal_with_sorted_columns(experiment_df, experiment_df_truth)

    @pytest.mark.usefixtures("get_context_not_found_mock")
    def test_get_experiment_df_not_exist(self):
        aiplatform.init(project=_TEST_PROJECT, location=_TEST_LOCATION)
        with pytest.raises(exceptions.NotFound):
            aiplatform.get_experiment_df(_TEST_EXPERIMENT)

    @pytest.mark.usefixtures("get_pipeline_context_mock")
    def test_get_experiment_df_wrong_schema(self):
        aiplatform.init(project=_TEST_PROJECT, location=_TEST_LOCATION)
        with pytest.raises(ValueError):
            aiplatform.get_experiment_df(_TEST_EXPERIMENT)



import pytest

from google.api_core import exceptions
from google.cloud import aiplatform
from google.cloud.aiplatform.vizier import Study
from google.cloud.aiplatform.vizier import Trial
from tests.system.aiplatform import e2e_base
from google.cloud.aiplatform.vizier import pyvizier

_TEST_STUDY_ID = 123


@pytest.mark.usefixtures("tear_down_resources")
class TestVizier(e2e_base.TestEndToEnd):
    _temp_prefix = "temp_vertex_sdk_e2e_vizier_test"

    def test_vizier_lifecycle(self, shared_state):
        aiplatform.init(
            project=e2e_base._PROJECT,
            location=e2e_base._LOCATION,
        )
        sc = pyvizier.StudyConfig()
        sc.algorithm = pyvizier.Algorithm.RANDOM_SEARCH
        sc.metric_information.append(
            pyvizier.MetricInformation(
                name="pr-auc", goal=pyvizier.ObjectiveMetricGoal.MAXIMIZE
            )
        )
        root = sc.search_space.select_root()
        root.add_float_param(
            "learning_rate", 0.00001, 1.0, scale_type=pyvizier.ScaleType.LINEAR
        )
        root.add_categorical_param("optimizer", ["adagrad", "adam", "experimental"])
        sc.automated_stopping_config = (
            pyvizier.AutomatedStoppingConfig.decay_curve_stopping_config(use_steps=True)
        )

        study = Study.create_or_load(display_name=self._temp_prefix, problem=sc)
        shared_state["resources"] = [study]
        trials = study.suggest(count=3, worker="halio_test_worker")
        for trial in trials:
            if not trial.should_stop():
                measurement = pyvizier.Measurement()
                measurement.metrics["pr-auc"] = 0.4
                trial.add_measurement(measurement=measurement)
                trial.complete(measurement=measurement)
        optimal_trials = study.optimal_trials()

        for trial in study.trials():
            assert trial.status == pyvizier.TrialStatus.COMPLETED
        assert optimal_trials[0].status == pyvizier.TrialStatus.COMPLETED

    def test_vizier_study_deletion(self, shared_state):
        aiplatform.init(
            project=e2e_base._PROJECT,
            location=e2e_base._LOCATION,
        )
        sc = pyvizier.StudyConfig()
        sc.algorithm = pyvizier.Algorithm.RANDOM_SEARCH
        sc.metric_information.append(
            pyvizier.MetricInformation(
                name="pr-auc", goal=pyvizier.ObjectiveMetricGoal.MAXIMIZE
            )
        )
        root = sc.search_space.select_root()
        root.add_float_param(
            "learning_rate", 0.00001, 1.0, scale_type=pyvizier.ScaleType.LINEAR
        )
        root.add_categorical_param("optimizer", ["adagrad", "adam", "experimental"])
        sc.automated_stopping_config = (
            pyvizier.AutomatedStoppingConfig.decay_curve_stopping_config(use_steps=True)
        )

        study = Study.create_or_load(display_name=self._temp_prefix, problem=sc)
        study.delete()

        with pytest.raises(exceptions.NotFound):
            study = Study(study_id=study.name)

    def test_vizier_trial_deletion(self, shared_state):
        aiplatform.init(
            project=e2e_base._PROJECT,
            location=e2e_base._LOCATION,
        )
        sc = pyvizier.StudyConfig()
        sc.algorithm = pyvizier.Algorithm.RANDOM_SEARCH
        sc.metric_information.append(
            pyvizier.MetricInformation(
                name="pr-auc", goal=pyvizier.ObjectiveMetricGoal.MAXIMIZE
            )
        )
        root = sc.search_space.select_root()
        root.add_float_param(
            "learning_rate", 0.00001, 1.0, scale_type=pyvizier.ScaleType.LINEAR
        )
        root.add_categorical_param("optimizer", ["adagrad", "adam", "experimental"])
        sc.automated_stopping_config = (
            pyvizier.AutomatedStoppingConfig.decay_curve_stopping_config(use_steps=True)
        )

        study = Study.create_or_load(display_name=self._temp_prefix, problem=sc)
        trials = study.suggest(count=1, worker="halio_test_worker")
        trials[0].delete()

        with pytest.raises(exceptions.NotFound):
            study = Trial(study_id=study.name, trial_name=trials[0].name)



# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from google.cloud.aiplatform import version as aiplatform_version

__version__ = aiplatform_version.__version__


from google.cloud.aiplatform import initializer

from google.cloud.aiplatform.datasets import (
    ImageDataset,
    TabularDataset,
    TextDataset,
    TimeSeriesDataset,
    VideoDataset,
)
from google.cloud.aiplatform import explain
from google.cloud.aiplatform import gapic
from google.cloud.aiplatform import hyperparameter_tuning
from google.cloud.aiplatform.featurestore import (
    EntityType,
    Feature,
    Featurestore,
)
from google.cloud.aiplatform.matching_engine import (
    MatchingEngineIndex,
    MatchingEngineIndexEndpoint,
)
from google.cloud.aiplatform import metadata
from google.cloud.aiplatform.models import Endpoint
from google.cloud.aiplatform.models import PrivateEndpoint
from google.cloud.aiplatform.models import Model
from google.cloud.aiplatform.models import ModelRegistry
from google.cloud.aiplatform.model_evaluation import ModelEvaluation
from google.cloud.aiplatform.jobs import (
    BatchPredictionJob,
    CustomJob,
    HyperparameterTuningJob,
    ModelDeploymentMonitoringJob,
)
from google.cloud.aiplatform.pipeline_jobs import PipelineJob
from google.cloud.aiplatform.tensorboard import (
    Tensorboard,
    TensorboardExperiment,
    TensorboardRun,
    TensorboardTimeSeries,
)
from google.cloud.aiplatform.training_jobs import (
    CustomTrainingJob,
    CustomContainerTrainingJob,
    CustomPythonPackageTrainingJob,
    AutoMLTabularTrainingJob,
    AutoMLForecastingTrainingJob,
    SequenceToSequencePlusForecastingTrainingJob,
    TemporalFusionTransformerForecastingTrainingJob,
    AutoMLImageTrainingJob,
    AutoMLTextTrainingJob,
    AutoMLVideoTrainingJob,
)

from google.cloud.aiplatform import helpers

"""
Usage:
from google.cloud import aiplatform

aiplatform.init(project='my_project')
"""
init = initializer.global_config.init

get_pipeline_df = metadata.metadata._LegacyExperimentService.get_pipeline_df

log_params = metadata.metadata._experiment_tracker.log_params
log_metrics = metadata.metadata._experiment_tracker.log_metrics
log_classification_metrics = (
    metadata.metadata._experiment_tracker.log_classification_metrics
)
log_model = metadata.metadata._experiment_tracker.log_model
get_experiment_df = metadata.metadata._experiment_tracker.get_experiment_df
start_run = metadata.metadata._experiment_tracker.start_run
autolog = metadata.metadata._experiment_tracker.autolog
start_execution = metadata.metadata._experiment_tracker.start_execution
log = metadata.metadata._experiment_tracker.log
log_time_series_metrics = metadata.metadata._experiment_tracker.log_time_series_metrics
end_run = metadata.metadata._experiment_tracker.end_run

save_model = metadata._models.save_model
get_experiment_model = metadata.schema.google.artifact_schema.ExperimentModel.get

Experiment = metadata.experiment_resources.Experiment
ExperimentRun = metadata.experiment_run_resource.ExperimentRun
Artifact = metadata.artifact.Artifact
Execution = metadata.execution.Execution
Context = metadata.context.Context


__all__ = (
    "end_run",
    "explain",
    "gapic",
    "init",
    "helpers",
    "hyperparameter_tuning",
    "log",
    "log_params",
    "log_metrics",
    "log_classification_metrics",
    "log_model",
    "log_time_series_metrics",
    "get_experiment_df",
    "get_pipeline_df",
    "start_run",
    "start_execution",
    "save_model",
    "get_experiment_model",
    "Artifact",
    "AutoMLImageTrainingJob",
    "AutoMLTabularTrainingJob",
    "AutoMLForecastingTrainingJob",
    "AutoMLTextTrainingJob",
    "AutoMLVideoTrainingJob",
    "BatchPredictionJob",
    "CustomJob",
    "CustomTrainingJob",
    "CustomContainerTrainingJob",
    "CustomPythonPackageTrainingJob",
    "EmailAlertConfig",
    "Endpoint",
    "DriftDetectionConfig",
    "ExplanationConfig",
    "ObjectiveConfig",
    "SkewDetectionConfig",
    "EntityType",
    "Execution",
    "Experiment",
    "ExperimentRun",
    "Feature",
    "Featurestore",
    "MatchingEngineIndex",
    "MatchingEngineIndexEndpoint",
    "ImageDataset",
    "HyperparameterTuningJob",
    "Model",
    "ModelRegistry",
    "ModelEvaluation",
    "ModelDeploymentMonitoringJob",
    "PipelineJob",
    "PrivateEndpoint",
    "RandomSampleConfig",
    "SequenceToSequencePlusForecastingTrainingJob",
    "ScheduleConfig",
    "TabularDataset",
    "Tensorboard",
    "TensorboardExperiment",
    "TensorboardRun",
    "TensorboardTimeSeries",
    "TextDataset",
    "TemporalFusionTransformerForecastingTrainingJob",
    "TimeSeriesDataset",
    "VideoDataset",
)



# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import pytest
import uuid

from unittest import mock
from importlib import reload
from unittest.mock import patch

from google.api_core import operation
from google.protobuf import field_mask_pb2

from google.cloud import aiplatform
from google.cloud.aiplatform import base
from google.cloud.aiplatform import initializer
from google.cloud.aiplatform.compat.services import (
    index_service_client,
)

from google.cloud.aiplatform.compat.types import index as gca_index

# project
_TEST_PROJECT = "test-project"
_TEST_LOCATION = "us-central1"
_TEST_PARENT = f"projects/{_TEST_PROJECT}/locations/{_TEST_LOCATION}"


# index
_TEST_INDEX_ID = "index_id"
_TEST_INDEX_NAME = f"{_TEST_PARENT}/indexes/{_TEST_INDEX_ID}"
_TEST_INDEX_DISPLAY_NAME = "index_display_name"
_TEST_CONTENTS_DELTA_URI = "gs://contents"
_TEST_INDEX_DISTANCE_MEASURE_TYPE = "SQUARED_L2_DISTANCE"

_TEST_CONTENTS_DELTA_URI_UPDATE = "gs://contents_update"
_TEST_IS_COMPLETE_OVERWRITE_UPDATE = True

_TEST_INDEX_CONFIG_DIMENSIONS = 100
_TEST_INDEX_APPROXIMATE_NEIGHBORS_COUNT = 150
_TEST_LEAF_NODE_EMBEDDING_COUNT = 123
_TEST_LEAF_NODES_TO_SEARCH_PERCENT = 50

_TEST_INDEX_DESCRIPTION = "index_description"


_TEST_LABELS = {"my_key": "my_value"}
_TEST_DISPLAY_NAME_UPDATE = "my new display name"
_TEST_DESCRIPTION_UPDATE = "my description update"
_TEST_LABELS_UPDATE = {"my_key_update": "my_value_update"}

# request_metadata
_TEST_REQUEST_METADATA = ()

# Lists
_TEST_INDEX_LIST = [
    gca_index.Index(
        name=_TEST_INDEX_NAME,
        display_name=_TEST_INDEX_DISPLAY_NAME,
        description=_TEST_INDEX_DESCRIPTION,
    ),
    gca_index.Index(
        name=_TEST_INDEX_NAME,
        display_name=_TEST_INDEX_DISPLAY_NAME,
        description=_TEST_INDEX_DESCRIPTION,
    ),
    gca_index.Index(
        name=_TEST_INDEX_NAME,
        display_name=_TEST_INDEX_DISPLAY_NAME,
        description=_TEST_INDEX_DESCRIPTION,
    ),
]


def uuid_mock():
    return uuid.UUID(int=1)


# All Index Mocks
@pytest.fixture
def get_index_mock():
    with patch.object(
        index_service_client.IndexServiceClient, "get_index"
    ) as get_index_mock:
        get_index_mock.return_value = gca_index.Index(
            name=_TEST_INDEX_NAME,
            display_name=_TEST_INDEX_DISPLAY_NAME,
            description=_TEST_INDEX_DESCRIPTION,
        )
        yield get_index_mock


@pytest.fixture
def update_index_metadata_mock():
    with patch.object(
        index_service_client.IndexServiceClient, "update_index"
    ) as update_index_mock:
        index_lro_mock = mock.Mock(operation.Operation)
        index_lro_mock.result.return_value = gca_index.Index(
            name=_TEST_INDEX_NAME,
            display_name=_TEST_DISPLAY_NAME_UPDATE,
            description=_TEST_DESCRIPTION_UPDATE,
            labels=_TEST_LABELS_UPDATE,
        )
        update_index_mock.return_value = index_lro_mock
        yield update_index_mock


@pytest.fixture
def update_index_embeddings_mock():
    with patch.object(
        index_service_client.IndexServiceClient, "update_index"
    ) as update_index_mock:
        index_lro_mock = mock.Mock(operation.Operation)
        index_lro_mock.result.return_value = gca_index.Index(
            name=_TEST_INDEX_NAME,
        )
        update_index_mock.return_value = index_lro_mock
        yield update_index_mock


@pytest.fixture
def list_indexes_mock():
    with patch.object(
        index_service_client.IndexServiceClient, "list_indexes"
    ) as list_indexes_mock:
        list_indexes_mock.return_value = _TEST_INDEX_LIST
        yield list_indexes_mock


@pytest.fixture
def delete_index_mock():
    with mock.patch.object(
        index_service_client.IndexServiceClient, "delete_index"
    ) as delete_index_mock:
        delete_index_lro_mock = mock.Mock(operation.Operation)
        delete_index_mock.return_value = delete_index_lro_mock
        yield delete_index_mock


@pytest.fixture
def create_index_mock():
    with patch.object(
        index_service_client.IndexServiceClient, "create_index"
    ) as create_index_mock:
        create_index_lro_mock = mock.Mock(operation.Operation)
        create_index_lro_mock.result.return_value = gca_index.Index(
            name=_TEST_INDEX_NAME,
            display_name=_TEST_INDEX_DISPLAY_NAME,
            description=_TEST_INDEX_DESCRIPTION,
        )
        create_index_mock.return_value = create_index_lro_mock
        yield create_index_mock


@pytest.mark.usefixtures("google_auth_mock")
class TestMatchingEngineIndex:
    def setup_method(self):
        reload(initializer)
        reload(aiplatform)

    def teardown_method(self):
        initializer.global_pool.shutdown(wait=True)

    @pytest.mark.parametrize("index_name", [_TEST_INDEX_ID, _TEST_INDEX_NAME])
    def test_init_index(self, index_name, get_index_mock):
        aiplatform.init(project=_TEST_PROJECT)

        my_index = aiplatform.MatchingEngineIndex(index_name=index_name)

        get_index_mock.assert_called_once_with(
            name=my_index.resource_name, retry=base._DEFAULT_RETRY
        )

    @pytest.mark.usefixtures("get_index_mock")
    def test_update_index_metadata(self, update_index_metadata_mock):
        aiplatform.init(project=_TEST_PROJECT)

        my_index = aiplatform.MatchingEngineIndex(index_name=_TEST_INDEX_ID)
        updated_index = my_index.update_metadata(
            display_name=_TEST_DISPLAY_NAME_UPDATE,
            description=_TEST_DESCRIPTION_UPDATE,
            labels=_TEST_LABELS_UPDATE,
        )

        expected = gca_index.Index(
            name=_TEST_INDEX_NAME,
            display_name=_TEST_DISPLAY_NAME_UPDATE,
            description=_TEST_DESCRIPTION_UPDATE,
            labels=_TEST_LABELS_UPDATE,
        )

        update_index_metadata_mock.assert_called_once_with(
            index=expected,
            update_mask=field_mask_pb2.FieldMask(
                paths=["labels", "display_name", "description"]
            ),
            metadata=_TEST_REQUEST_METADATA,
        )

        assert updated_index.gca_resource == expected

    @pytest.mark.usefixtures("get_index_mock")
    def test_update_index_embeddings(self, update_index_embeddings_mock):
        aiplatform.init(project=_TEST_PROJECT)

        my_index = aiplatform.MatchingEngineIndex(index_name=_TEST_INDEX_ID)
        updated_index = my_index.update_embeddings(
            contents_delta_uri=_TEST_CONTENTS_DELTA_URI_UPDATE,
            is_complete_overwrite=_TEST_IS_COMPLETE_OVERWRITE_UPDATE,
        )

        expected = gca_index.Index(
            name=_TEST_INDEX_NAME,
            metadata={
                "contentsDeltaUri": _TEST_CONTENTS_DELTA_URI_UPDATE,
                "isCompleteOverwrite": _TEST_IS_COMPLETE_OVERWRITE_UPDATE,
            },
        )

        update_index_embeddings_mock.assert_called_once_with(
            index=expected,
            update_mask=field_mask_pb2.FieldMask(paths=["metadata"]),
            metadata=_TEST_REQUEST_METADATA,
        )

        # The service only returns the name of the Index
        assert updated_index.gca_resource == gca_index.Index(name=_TEST_INDEX_NAME)

    def test_list_indexes(self, list_indexes_mock):
        aiplatform.init(project=_TEST_PROJECT)

        my_indexes_list = aiplatform.MatchingEngineIndex.list()

        list_indexes_mock.assert_called_once_with(request={"parent": _TEST_PARENT})
        assert len(my_indexes_list) == len(_TEST_INDEX_LIST)
        for my_index in my_indexes_list:
            assert type(my_index) == aiplatform.MatchingEngineIndex

    @pytest.mark.parametrize("sync", [True, False])
    @pytest.mark.usefixtures("get_index_mock")
    def test_delete_index(self, delete_index_mock, sync):
        aiplatform.init(project=_TEST_PROJECT)

        my_index = aiplatform.MatchingEngineIndex(index_name=_TEST_INDEX_ID)
        my_index.delete(sync=sync)

        if not sync:
            my_index.wait()

        delete_index_mock.assert_called_once_with(name=my_index.resource_name)

    @pytest.mark.usefixtures("get_index_mock")
    @pytest.mark.parametrize("sync", [True, False])
    def test_create_tree_ah_index(self, create_index_mock, sync):
        aiplatform.init(project=_TEST_PROJECT)

        my_index = aiplatform.MatchingEngineIndex.create_tree_ah_index(
            display_name=_TEST_INDEX_DISPLAY_NAME,
            contents_delta_uri=_TEST_CONTENTS_DELTA_URI,
            dimensions=_TEST_INDEX_CONFIG_DIMENSIONS,
            approximate_neighbors_count=_TEST_INDEX_APPROXIMATE_NEIGHBORS_COUNT,
            distance_measure_type=_TEST_INDEX_DISTANCE_MEASURE_TYPE,
            leaf_node_embedding_count=_TEST_LEAF_NODE_EMBEDDING_COUNT,
            leaf_nodes_to_search_percent=_TEST_LEAF_NODES_TO_SEARCH_PERCENT,
            description=_TEST_INDEX_DESCRIPTION,
            labels=_TEST_LABELS,
            sync=sync,
        )

        if not sync:
            my_index.wait()

        config = {
            "treeAhConfig": {
                "leafNodeEmbeddingCount": _TEST_LEAF_NODE_EMBEDDING_COUNT,
                "leafNodesToSearchPercent": _TEST_LEAF_NODES_TO_SEARCH_PERCENT,
            }
        }

        expected = gca_index.Index(
            display_name=_TEST_INDEX_DISPLAY_NAME,
            metadata={
                "config": {
                    "algorithmConfig": config,
                    "dimensions": _TEST_INDEX_CONFIG_DIMENSIONS,
                    "approximateNeighborsCount": _TEST_INDEX_APPROXIMATE_NEIGHBORS_COUNT,
                    "distanceMeasureType": _TEST_INDEX_DISTANCE_MEASURE_TYPE,
                },
                "contentsDeltaUri": _TEST_CONTENTS_DELTA_URI,
            },
            description=_TEST_INDEX_DESCRIPTION,
            labels=_TEST_LABELS,
        )

        create_index_mock.assert_called_once_with(
            parent=_TEST_PARENT,
            index=expected,
            metadata=_TEST_REQUEST_METADATA,
        )

    @pytest.mark.usefixtures("get_index_mock")
    @pytest.mark.parametrize("sync", [True, False])
    def test_create_brute_force_index(self, create_index_mock, sync):
        aiplatform.init(project=_TEST_PROJECT)

        my_index = aiplatform.MatchingEngineIndex.create_brute_force_index(
            display_name=_TEST_INDEX_DISPLAY_NAME,
            contents_delta_uri=_TEST_CONTENTS_DELTA_URI,
            dimensions=_TEST_INDEX_CONFIG_DIMENSIONS,
            distance_measure_type=_TEST_INDEX_DISTANCE_MEASURE_TYPE,
            description=_TEST_INDEX_DESCRIPTION,
            labels=_TEST_LABELS,
            sync=sync,
        )

        if not sync:
            my_index.wait()

        config = {"bruteForceConfig": {}}

        expected = gca_index.Index(
            display_name=_TEST_INDEX_DISPLAY_NAME,
            metadata={
                "config": {
                    "algorithmConfig": config,
                    "dimensions": _TEST_INDEX_CONFIG_DIMENSIONS,
                    "approximateNeighborsCount": None,
                    "distanceMeasureType": _TEST_INDEX_DISTANCE_MEASURE_TYPE,
                },
                "contentsDeltaUri": _TEST_CONTENTS_DELTA_URI,
            },
            description=_TEST_INDEX_DESCRIPTION,
            labels=_TEST_LABELS,
        )

        create_index_mock.assert_called_once_with(
            parent=_TEST_PARENT,
            index=expected,
            metadata=_TEST_REQUEST_METADATA,
        )



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.cloud.aiplatform.v1beta1.schema.trainingjob.definition_v1beta1 import (
    gapic_version as package_version,
)

__version__ = package_version.__version__


from .types.automl_image_classification import AutoMlImageClassification
from .types.automl_image_classification import AutoMlImageClassificationInputs
from .types.automl_image_classification import AutoMlImageClassificationMetadata
from .types.automl_image_object_detection import AutoMlImageObjectDetection
from .types.automl_image_object_detection import AutoMlImageObjectDetectionInputs
from .types.automl_image_object_detection import AutoMlImageObjectDetectionMetadata
from .types.automl_image_segmentation import AutoMlImageSegmentation
from .types.automl_image_segmentation import AutoMlImageSegmentationInputs
from .types.automl_image_segmentation import AutoMlImageSegmentationMetadata
from .types.automl_tables import AutoMlTables
from .types.automl_tables import AutoMlTablesInputs
from .types.automl_tables import AutoMlTablesMetadata
from .types.automl_text_classification import AutoMlTextClassification
from .types.automl_text_classification import AutoMlTextClassificationInputs
from .types.automl_text_extraction import AutoMlTextExtraction
from .types.automl_text_extraction import AutoMlTextExtractionInputs
from .types.automl_text_sentiment import AutoMlTextSentiment
from .types.automl_text_sentiment import AutoMlTextSentimentInputs
from .types.automl_time_series_forecasting import AutoMlForecasting
from .types.automl_time_series_forecasting import AutoMlForecastingInputs
from .types.automl_time_series_forecasting import AutoMlForecastingMetadata
from .types.automl_video_action_recognition import AutoMlVideoActionRecognition
from .types.automl_video_action_recognition import AutoMlVideoActionRecognitionInputs
from .types.automl_video_classification import AutoMlVideoClassification
from .types.automl_video_classification import AutoMlVideoClassificationInputs
from .types.automl_video_object_tracking import AutoMlVideoObjectTracking
from .types.automl_video_object_tracking import AutoMlVideoObjectTrackingInputs
from .types.export_evaluated_data_items_config import ExportEvaluatedDataItemsConfig

__all__ = (
    "AutoMlForecasting",
    "AutoMlForecastingInputs",
    "AutoMlForecastingMetadata",
    "AutoMlImageClassification",
    "AutoMlImageClassificationInputs",
    "AutoMlImageClassificationMetadata",
    "AutoMlImageObjectDetection",
    "AutoMlImageObjectDetectionInputs",
    "AutoMlImageObjectDetectionMetadata",
    "AutoMlImageSegmentation",
    "AutoMlImageSegmentationInputs",
    "AutoMlImageSegmentationMetadata",
    "AutoMlTables",
    "AutoMlTablesInputs",
    "AutoMlTablesMetadata",
    "AutoMlTextClassification",
    "AutoMlTextClassificationInputs",
    "AutoMlTextExtraction",
    "AutoMlTextExtractionInputs",
    "AutoMlTextSentiment",
    "AutoMlTextSentimentInputs",
    "AutoMlVideoActionRecognition",
    "AutoMlVideoActionRecognitionInputs",
    "AutoMlVideoClassification",
    "AutoMlVideoClassificationInputs",
    "AutoMlVideoObjectTracking",
    "AutoMlVideoObjectTrackingInputs",
    "ExportEvaluatedDataItemsConfig",
)



# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import io
import os

import setuptools  # type: ignore

name = "google-cloud-aiplatform"
description = "Vertex AI API client library"

package_root = os.path.abspath(os.path.dirname(__file__))
readme_filename = os.path.join(package_root, "README.rst")
with io.open(readme_filename, encoding="utf-8") as readme_file:
    readme = readme_file.read()

version = {}
with open(os.path.join(package_root, "google/cloud/aiplatform/version.py")) as fp:
    exec(fp.read(), version)
version = version["__version__"]

tensorboard_extra_require = ["tensorflow >=2.3.0, <3.0.0dev"]
metadata_extra_require = ["pandas >= 1.0.0", "numpy>=1.15.0"]
xai_extra_require = ["tensorflow >=2.3.0, <3.0.0dev"]
lit_extra_require = [
    "tensorflow >= 2.3.0, <3.0.0dev",
    "pandas >= 1.0.0",
    "lit-nlp == 0.4.0",
    "explainable-ai-sdk >= 1.0.0",
]
profiler_extra_require = [
    "tensorboard-plugin-profile >= 2.4.0, <3.0.0dev",
    "werkzeug >= 2.0.0, <2.1.0dev",
    "tensorflow >=2.4.0, <3.0.0dev",
]
featurestore_extra_require = [
    "google-cloud-bigquery-storage",
    "pandas >= 1.0.0",
    "pyarrow >= 6.0.1",
]
pipelines_extra_require = [
    "pyyaml>=5.3,<6",
]
datasets_extra_require = [
    "pyarrow >= 3.0.0, < 8.0dev",
]

vizier_extra_require = [
    "google-vizier==0.0.4",
]

prediction_extra_require = [
    "docker >= 5.0.3",
    # TODO: remove the upper bound after a new version is released.
    #   See https://github.com/tiangolo/fastapi/pull/4488.
    "fastapi >= 0.71.0, <0.76.0",
    "starlette >= 0.17.1",
    "uvicorn[standard] >= 0.16.0",
]

endpoint_extra_require = ["requests >= 2.28.1"]

private_endpoints_extra_require = ["urllib3 >=1.21.1, <1.27", "requests >= 2.28.1"]

autologging_extra_require = ["mlflow>=1.27.0,<=2.1.1"]

full_extra_require = list(
    set(
        tensorboard_extra_require
        + metadata_extra_require
        + xai_extra_require
        + lit_extra_require
        + featurestore_extra_require
        + pipelines_extra_require
        + datasets_extra_require
        + endpoint_extra_require
        + vizier_extra_require
        + prediction_extra_require
        + private_endpoints_extra_require
        + autologging_extra_require
    )
)
testing_extra_require = (
    full_extra_require
    + profiler_extra_require
    + [
        "grpcio-testing",
        "pytest-asyncio",
        "pytest-xdist",
        "ipython",
        "kfp",
        "xgboost",
        "scikit-learn",
    ]
)


setuptools.setup(
    name=name,
    version=version,
    description=description,
    long_description=readme,
    packages=[
        package
        for package in setuptools.PEP420PackageFinder.find()
        if package.startswith("google")
    ],
    entry_points={
        "console_scripts": [
            "tb-gcp-uploader=google.cloud.aiplatform.tensorboard.uploader_main:run_main"
        ],
    },
    namespace_packages=("google", "google.cloud"),
    author="Google LLC",
    author_email="googleapis-packages@google.com",
    license="Apache 2.0",
    url="https://github.com/googleapis/python-aiplatform",
    platforms="Posix; MacOS X; Windows",
    include_package_data=True,
    install_requires=(
        "google-api-core[grpc] >= 1.32.0, <3.0.0dev,!=2.0.*,!=2.1.*,!=2.2.*,!=2.3.*,!=2.4.*,!=2.5.*,!=2.6.*,!=2.7.*",
        "proto-plus >= 1.22.0, <2.0.0dev",
        "protobuf>=3.19.5,<5.0.0dev,!=3.20.0,!=3.20.1,!=4.21.0,!=4.21.1,!=4.21.2,!=4.21.3,!=4.21.4,!=4.21.5",
        "packaging >= 14.3, <22.0.0dev",
        "google-cloud-storage >= 1.32.0, < 3.0.0dev",
        "google-cloud-bigquery >= 1.15.0, < 4.0.0dev",
        "google-cloud-resource-manager >= 1.3.3, < 3.0.0dev",
        "shapely < 2.0.0",
    ),
    extras_require={
        "endpoint": endpoint_extra_require,
        "full": full_extra_require,
        "metadata": metadata_extra_require,
        "tensorboard": tensorboard_extra_require,
        "testing": testing_extra_require,
        "xai": xai_extra_require,
        "lit": lit_extra_require,
        "cloud_profiler": profiler_extra_require,
        "pipelines": pipelines_extra_require,
        "vizier": vizier_extra_require,
        "prediction": prediction_extra_require,
        "datasets": datasets_extra_require,
        "private_endpoints": private_endpoints_extra_require,
        "autologging": autologging_extra_require,
    },
    python_requires=">=3.7",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    zip_safe=False,
)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import MutableMapping, MutableSequence

import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.aiplatform.v1",
    manifest={
        "AvroSource",
        "CsvSource",
        "GcsSource",
        "GcsDestination",
        "BigQuerySource",
        "BigQueryDestination",
        "CsvDestination",
        "TFRecordDestination",
        "ContainerRegistryDestination",
    },
)


class AvroSource(proto.Message):
    r"""The storage details for Avro input content.

    Attributes:
        gcs_source (google.cloud.aiplatform_v1.types.GcsSource):
            Required. Google Cloud Storage location.
    """

    gcs_source: "GcsSource" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="GcsSource",
    )


class CsvSource(proto.Message):
    r"""The storage details for CSV input content.

    Attributes:
        gcs_source (google.cloud.aiplatform_v1.types.GcsSource):
            Required. Google Cloud Storage location.
    """

    gcs_source: "GcsSource" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="GcsSource",
    )


class GcsSource(proto.Message):
    r"""The Google Cloud Storage location for the input content.

    Attributes:
        uris (MutableSequence[str]):
            Required. Google Cloud Storage URI(-s) to the
            input file(s). May contain wildcards. For more
            information on wildcards, see
            https://cloud.google.com/storage/docs/gsutil/addlhelp/WildcardNames.
    """

    uris: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=1,
    )


class GcsDestination(proto.Message):
    r"""The Google Cloud Storage location where the output is to be
    written to.

    Attributes:
        output_uri_prefix (str):
            Required. Google Cloud Storage URI to output
            directory. If the uri doesn't end with
            '/', a '/' will be automatically appended. The
            directory is created if it doesn't exist.
    """

    output_uri_prefix: str = proto.Field(
        proto.STRING,
        number=1,
    )


class BigQuerySource(proto.Message):
    r"""The BigQuery location for the input content.

    Attributes:
        input_uri (str):
            Required. BigQuery URI to a table, up to 2000 characters
            long. Accepted forms:

            -  BigQuery path. For example:
               ``bq://projectId.bqDatasetId.bqTableId``.
    """

    input_uri: str = proto.Field(
        proto.STRING,
        number=1,
    )


class BigQueryDestination(proto.Message):
    r"""The BigQuery location for the output content.

    Attributes:
        output_uri (str):
            Required. BigQuery URI to a project or table, up to 2000
            characters long.

            When only the project is specified, the Dataset and Table is
            created. When the full table reference is specified, the
            Dataset must exist and table must not exist.

            Accepted forms:

            -  BigQuery path. For example: ``bq://projectId`` or
               ``bq://projectId.bqDatasetId`` or
               ``bq://projectId.bqDatasetId.bqTableId``.
    """

    output_uri: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CsvDestination(proto.Message):
    r"""The storage details for CSV output content.

    Attributes:
        gcs_destination (google.cloud.aiplatform_v1.types.GcsDestination):
            Required. Google Cloud Storage location.
    """

    gcs_destination: "GcsDestination" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="GcsDestination",
    )


class TFRecordDestination(proto.Message):
    r"""The storage details for TFRecord output content.

    Attributes:
        gcs_destination (google.cloud.aiplatform_v1.types.GcsDestination):
            Required. Google Cloud Storage location.
    """

    gcs_destination: "GcsDestination" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="GcsDestination",
    )


class ContainerRegistryDestination(proto.Message):
    r"""The Container Registry location for the container image.

    Attributes:
        output_uri (str):
            Required. Container Registry URI of a container image. Only
            Google Container Registry and Artifact Registry are
            supported now. Accepted forms:

            -  Google Container Registry path. For example:
               ``gcr.io/projectId/imageName:tag``.

            -  Artifact Registry path. For example:
               ``us-central1-docker.pkg.dev/projectId/repoName/imageName:tag``.

            If a tag is not specified, "latest" will be used as the
            default tag.
    """

    output_uri: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from abc import ABC, abstractmethod
import json
from typing import Any, Optional

try:
    from fastapi import HTTPException
except ImportError:
    raise ImportError(
        "FastAPI is not installed and is required to build model servers. "
        'Please install the SDK using `pip install "google-cloud-aiplatform[prediction]>=1.16.0"`.'
    )

from google.cloud.aiplatform.constants import prediction as prediction_constants
from google.cloud.aiplatform.prediction import handler_utils


APPLICATION_JSON = "application/json"


class Serializer(ABC):
    """Interface to implement serialization and deserialization for prediction."""

    @staticmethod
    @abstractmethod
    def deserialize(data: Any, content_type: Optional[str]) -> Any:
        """Deserializes the request data. Invoked before predict.

        Args:
            data (Any):
                Required. The request data sent to the application.
            content_type (str):
                Optional. The specified content type of the request.
        """
        pass

    @staticmethod
    @abstractmethod
    def serialize(prediction: Any, accept: Optional[str]) -> Any:
        """Serializes the prediction results. Invoked after predict.

        Args:
            prediction (Any):
                Required. The generated prediction to be sent back to clients.
            accept (str):
                Optional. The specified content type of the response.
        """
        pass


class DefaultSerializer(Serializer):
    """Default serializer for serialization and deserialization for prediction."""

    @staticmethod
    def deserialize(data: Any, content_type: Optional[str]) -> Any:
        """Deserializes the request data. Invoked before predict.

        Args:
            data (Any):
                Required. The request data sent to the application.
            content_type (str):
                Optional. The specified content type of the request.

        Raises:
            HTTPException: If Json deserialization failed or the specified content type is not
                supported.
        """
        if content_type == APPLICATION_JSON:
            try:
                return json.loads(data)
            except json.JSONDecodeError:
                raise HTTPException(
                    status_code=400,
                    detail=(
                        f"JSON deserialization failed for the request data: {data}.\n"
                        'To specify a different type, please set the "content-type" header '
                        "in the request.\nCurrently supported content-type in DefaultSerializer: "
                        f'"{APPLICATION_JSON}".'
                    ),
                )
        else:
            raise HTTPException(
                status_code=400,
                detail=(
                    f"Unsupported content type of the request: {content_type}.\n"
                    f'Currently supported content-type in DefaultSerializer: "{APPLICATION_JSON}".'
                ),
            )

    @staticmethod
    def serialize(prediction: Any, accept: Optional[str]) -> Any:
        """Serializes the prediction results. Invoked after predict.

        Args:
            prediction (Any):
                Required. The generated prediction to be sent back to clients.
            accept (str):
                Optional. The specified content type of the response.

        Raises:
            HTTPException: If Json serialization failed or the specified accept is not supported.
        """
        accept_dict = handler_utils.parse_accept_header(accept)

        if (
            APPLICATION_JSON in accept_dict
            or prediction_constants.ANY_ACCEPT_TYPE in accept_dict
        ):
            try:
                return json.dumps(prediction)
            except TypeError:
                raise HTTPException(
                    status_code=400,
                    detail=(
                        f"JSON serialization failed for the prediction result: {prediction}.\n"
                        'To specify a different type, please set the "accept" header '
                        "in the request.\nCurrently supported accept in DefaultSerializer: "
                        f'"{APPLICATION_JSON}".'
                    ),
                )
        else:
            raise HTTPException(
                status_code=400,
                detail=(
                    f"Unsupported accept of the response: {accept}.\n"
                    f'Currently supported accept in DefaultSerializer: "{APPLICATION_JSON}".'
                ),
            )
